import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.

from Products.ATContentTypes.tests import atcttestcase
from Products.ATContentTypes.config import TOOLNAME
from Products.ATContentTypes.config import SWALLOW_IMAGE_RESIZE_EXCEPTIONS
from Products.ATContentTypes.tool.atct import ATCTTool
from Products.CMFCore.utils import getToolByName

tests = []


class TestInstallation(atcttestcase.ATCTSiteTestCase):

    def afterSetUp(self):
        self.tool = getattr(self.portal.aq_explicit, TOOLNAME)
        self.ttool = getattr(self.portal.aq_explicit, 'portal_types')
        self.cat = getattr(self.portal.aq_explicit, 'portal_catalog')

    def test_tool_installed(self):
        t = getToolByName(self.portal, TOOLNAME, None)
        self.assertTrue(t, t)
        self.assertTrue(isinstance(t, ATCTTool), t.__class__)
        self.assertEqual(t.meta_type, 'ATCT Tool')
        self.assertEqual(t.getId(), TOOLNAME)

    def test_skin_installed(self):
        stool = getattr(self.portal.aq_explicit, 'portal_skins')
        self.assertTrue('ATContentTypes' in stool)

    def test_installedAllTypes(self):
        # test that all types are installed well
        ids = ('Document', 'File', 'Folder', 'Image', 'Link',
            'News Item', 'Topic', 'Event')
        for i in ids:
            self.assertTrue(i in self.ttool)

    def test_not_quickinstalled(self):
        qi = getattr(self.portal, 'portal_quickinstaller')
        products = [prod['id'] for prod in qi.listInstalledProducts()]
        self.assertFalse('ATContentTypes' in products)

    def test_release_settings_SAVE_TO_FAIL_FOR_DEVELOPMENT(self):
        self.assertEqual(SWALLOW_IMAGE_RESIZE_EXCEPTIONS, True)

    def test_reindex_doesnt_add_tools(self):
        cat = self.cat
        ids = [i for i in self.portal if i.startswith('portal_')]
        # a rought guess
        self.assertFalse(len(ids) < 5)
        for id in ids:
                result = cat(id=id)
                l = len(result)
                self.assertEqual(l, 0, (id, l, result))

    def test_adds_related_items_catalog_index(self):
        self.assertEqual(self.cat.Indexes['getRawRelatedItems'].__class__.__name__,
                         'KeywordIndex')

    def test_api_import(self):
        import Products.ATContentTypes.atct


tests.append(TestInstallation)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from Products.CMFCore.permissions import setDefaultRoles
from Products.Archetypes.atapi import listTypes
from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.interfaces import IATTopic
from Products.ATContentTypes.interfaces import IATTopicCriterion

TYPE_ROLES = ('Manager', 'Site Administrator', 'Owner')
TOPIC_ROLES = ('Manager', 'Site Administrator')
CHANGE_TOPIC_ROLES = TOPIC_ROLES + ('Owner',)
CRITERION_ROLES = ('Manager', 'Site Administrator')

# Gathering Topic and Event related permissions into one place
AddTopics = 'Add portal topics'
setDefaultRoles(AddTopics, TOPIC_ROLES)

ChangeTopics = 'Change portal topics'
setDefaultRoles(ChangeTopics, CHANGE_TOPIC_ROLES)

ChangeEvents = 'Change portal events'
setDefaultRoles(ChangeEvents, ('Manager', 'Site Administrator', 'Owner',))

ModifyConstrainTypes = "Modify constrain types"
setDefaultRoles(ModifyConstrainTypes, ('Manager', 'Site Administrator', 'Owner'))

ModifyViewTemplate = "Modify view template"
setDefaultRoles(ModifyViewTemplate, ('Manager', 'Site Administrator', 'Owner'))

ViewHistory = "ATContentTypes: View history"
setDefaultRoles(ViewHistory, ('Manager', 'Site Administrator'))

UploadViaURL = "ATContentTypes: Upload via url"
setDefaultRoles(UploadViaURL, ('Manager', 'Site Administrator'))

permissions = {}


def wireAddPermissions():
    """Creates a list of add permissions for all types in this project

    Must be called **after** all types are registered!
    """
    global permissions
    atct_types = listTypes(PROJECTNAME)
    for atct in atct_types:
        if IATTopic.implementedBy(atct['klass']):
            permission = AddTopics
        elif IATTopicCriterion.implementedBy(atct['klass']):
            permission = "%s Topic: Add %s" % (PROJECTNAME, atct['portal_type'])
            setDefaultRoles(permission, CRITERION_ROLES)
        else:
            permission = "%s: Add %s" % (PROJECTNAME, atct['portal_type'])
            setDefaultRoles(permission, TYPE_ROLES)

        permissions[atct['portal_type']] = permission
    return permissions



## Script (Python) "getValidIndexes"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=index
##title=Determine whether to show an id in an edit form

return context.allowedCriteriaForField(index, display_list=True)



from zope.interface import implements

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Acquisition import aq_parent
from Acquisition import aq_inner

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.PortalFolder import PortalFolderBase as PortalFolder

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import IntegerField
from Products.Archetypes.atapi import MultiSelectionWidget
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import IntDisplayList
from Products.Archetypes.atapi import DisplayList

from Products.ATContentTypes import ATCTMessageFactory as _
from Products.ATContentTypes import permission as ATCTPermissions

from Products.ATContentTypes.interfaces import ISelectableConstrainTypes

# constants for enableConstrainMixin
ACQUIRE = -1  # acquire locallyAllowedTypes from parent (default)
DISABLED = 0  # use default behavior of PortalFolder which uses the FTI information
ENABLED = 1  # allow types from locallyAllowedTypes only

# Note: ACQUIRED means get allowable types from parent (regardless of
#  whether it supports IConstrainTypes) but only if parent is the same
#  portal_type (folder within folder). Otherwise, use the global_allow/default
#  behaviour (same as DISABLED).

enableDisplayList = IntDisplayList((
    (ACQUIRE, _(u'constraintypes_acquire_label', default=u'Use parent folder settings')),
    (DISABLED, _(u'constraintypes_disable_label', default=u'Use portal default')),
    (ENABLED, _(u'constraintypes_enable_label', default=u'Select manually')),
    ))

ConstrainTypesMixinSchema = Schema((
    IntegerField('constrainTypesMode',
        required=False,
        default_method="_ct_defaultConstrainTypesMode",
        vocabulary=enableDisplayList,
        languageIndependent=True,
        write_permission=ATCTPermissions.ModifyConstrainTypes,
        widget=SelectionWidget(
            label=_(u'label_contrain_types_mode',
                    default=u'Constrain types mode'),
            description=_(u'description_constrain_types_mode',
                          default=u'Select the constraint type mode for this folder.'),
            visible={'view': 'invisible',
                     'edit': 'invisible',
                    },
            ),
        ),

    LinesField('locallyAllowedTypes',
        vocabulary='_ct_vocabularyPossibleTypes',
        enforceVocabulary=False,
        languageIndependent=True,
        default_method='_ct_defaultAddableTypeIds',
        accessor='getLocallyAllowedTypes',  # Respects ENABLE/DISABLE/ACQUIRE
        write_permission=ATCTPermissions.ModifyConstrainTypes,
        multiValued=True,
        widget=MultiSelectionWidget(
            size=10,
            label=_(u'label_constrain_allowed_types',
                    default=u'Permitted types'),
            description=_(u'description_constrain_allowed_types',
                          default=u'Select the types which will be addable inside this folder.'
                         ),
            visible={'view': 'invisible',
                     'edit': 'invisible',
                    },
            ),
        ),

     LinesField('immediatelyAddableTypes',
        vocabulary='_ct_vocabularyPossibleTypes',
        enforceVocabulary=False,
        languageIndependent=True,
        default_method='_ct_defaultAddableTypeIds',
        accessor='getImmediatelyAddableTypes',  # Respects ENABLE/DISABLE/ACQUIRE
        write_permission=ATCTPermissions.ModifyConstrainTypes,
        multiValued=True,
        widget=MultiSelectionWidget(
            size=10,
            label=_(u'label_constrain_preferred_types', u'Preferred types'),
            description=_(u'description_constrain_preferred_types',
                          default=u'Select the types which will be addable '
                                   'from the "Add new item" menu. Any '
                                   'additional types set in the list above '
                                   'will be addable from a separate form.'),
            visible={'view': 'invisible',
                     'edit': 'invisible',
                    },
            ),
        ),
    ))


def getParent(obj):
    portal_factory = getToolByName(obj, 'portal_factory', None)
    if portal_factory is not None and portal_factory.isTemporary(obj):
        # created by portal_factory
        parent = aq_parent(aq_parent(aq_parent(aq_inner(obj))))
    else:
        parent = aq_parent(aq_inner(obj))

    return parent


def parentPortalTypeEqual(obj):
    """Compares the portal type of obj to the portal type of its parent

    Return values:
        None - no acquisition context / parent available
        False - unequal
        True - equal
    """
    parent = getParent(obj)
    if parent is None:
        return None  # no context
    parent_type = getattr(parent.aq_explicit, 'portal_type', None)
    obj_type = getattr(obj.aq_explicit, 'portal_type')
    if obj_type and parent_type == obj_type:
        return True
    return False


class ConstrainTypesMixin:
    """ Gives the user with given rights the possibility to
        constrain the addable types on a per-folder basis.
    """

    implements(ISelectableConstrainTypes)

    security = ClassSecurityInfo()

    #
    # Sanity validator
    #
    security.declareProtected(ModifyPortalContent, 'validate_preferredTypes')
    def validate_preferredTypes(self, value):
        """Ensure that the preferred types is a subset of the allowed types.
        """
        allowed = self.getField('locallyAllowedTypes').get(self)
        preferred = value.split('\n')

        disallowed = []
        for p in preferred:
            if not p in allowed:
                disallowed.append(p)

        if disallowed:
            return "The following types are not permitted: %s" % \
                        ','.join(disallowed)

    #
    # Overrides + supplements for CMF types machinery
    #

    security.declareProtected(View, 'getLocallyAllowedTypes')
    def getLocallyAllowedTypes(self, context=None):
        """If enableTypeRestrictions is ENABLE, return the list of types
        set. If it is ACQUIRE, get the types set on the parent so long
        as the parent is of the same type - if not, use the same behaviuor as
        DISABLE: return the types allowable in the item.
        """
        if context is None:
            context = self
        mode = self.getConstrainTypesMode()

        if mode == DISABLED:
            return [fti.getId() for fti in self.getDefaultAddableTypes(context)]
        elif mode == ENABLED:
            return self.getField('locallyAllowedTypes').get(self)
        elif mode == ACQUIRE:
            parent = getParent(self)
            if not parent or parent.portal_type == 'Plone Site':
                return [fti.getId() for fti in self.getDefaultAddableTypes(context)]
            elif not parentPortalTypeEqual(self):
                # if parent.portal_type != self.portal_type:
                default_addable_types = [fti.getId() for fti in self.getDefaultAddableTypes(context)]
                if ISelectableConstrainTypes.providedBy(parent):
                    return [t for t in parent.getLocallyAllowedTypes(context)
                                if t in default_addable_types]
                else:
                    return [t for t in parent.getLocallyAllowedTypes()
                                if t in default_addable_types]
            else:
                if ISelectableConstrainTypes.providedBy(parent):
                    return parent.getLocallyAllowedTypes(context)
                else:
                    return parent.getLocallyAllowedTypes()
        else:
            raise ValueError, "Invalid value for enableAddRestriction"

    security.declareProtected(View, 'getImmediatelyAddableTypes')
    def getImmediatelyAddableTypes(self, context=None):
        """Get the list of type ids which should be immediately addable.
        If enableTypeRestrictions is ENABLE, return the list set; if it is
        ACQUIRE, use the value from the parent; if it is DISABLE, return
        all type ids allowable on the item.
        """
        if context is None:
            context = self
        mode = self.getConstrainTypesMode()

        if mode == DISABLED:
            return [fti.getId() for fti in \
                        self.getDefaultAddableTypes(context)]
        elif mode == ENABLED:
            return self.getField('immediatelyAddableTypes').get(self)
        elif mode == ACQUIRE:
            parent = getParent(self)
            if not parent or parent.portal_type == 'Plone Site':
                return [fti.getId() for fti in \
                        PortalFolder.allowedContentTypes(self)]
            elif not parentPortalTypeEqual(self):
                default_allowed = [fti.getId() for fti in \
                        PortalFolder.allowedContentTypes(self)]
                return [t for t in parent.getImmediatelyAddableTypes(context) \
                           if t in default_allowed]
            else:
                parent = aq_parent(aq_inner(self))
                return parent.getImmediatelyAddableTypes(context)
        else:
            raise ValueError, "Invalid value for enableAddRestriction"

    # overrides CMFCore's PortalFolder allowedTypes
    def allowedContentTypes(self, context=None):
        """returns constrained allowed types as list of fti's
        """
        if context is None:
            context = self
        mode = self.getConstrainTypesMode()

        # Short circuit if we are disabled or acquiring from non-compatible
        # parent

        parent = getParent(self)
        if mode == DISABLED or (mode == ACQUIRE and not parent):
            return PortalFolder.allowedContentTypes(self)
        elif mode == ACQUIRE and not parentPortalTypeEqual(self):
            globalTypes = self.getDefaultAddableTypes(context)
            if parent.portal_type == 'Plone Site':
                return globalTypes
            else:
                allowed = list(parent.getLocallyAllowedTypes(context))
                return [fti for fti in globalTypes if fti.getId() in allowed]
        else:
            globalTypes = self.getDefaultAddableTypes(context)
            allowed = list(self.getLocallyAllowedTypes())
            ftis = [fti for fti in globalTypes if fti.getId() in allowed]
            return ftis

    # overrides CMFCore's PortalFolder invokeFactory
    security.declareProtected(AddPortalContent, 'invokeFactory')
    def invokeFactory(self, type_name, id, RESPONSE=None, *args, **kw):
        """Invokes the portal_types tool
        """
        mode = self.getConstrainTypesMode()

        # Short circuit if we are disabled or acquiring from non-compatible
        # parent

        #if mode == DISABLED or \
        #        (parent and parent.portal_types != self.portal_types):
        if mode == DISABLED or \
              (mode == ACQUIRE and not parentPortalTypeEqual(self)):
            return PortalFolder.invokeFactory(self, type_name, id,
                                                RESPONSE=None, *args, **kw)

        if not type_name in [fti.getId() for fti in self.allowedContentTypes()]:
            raise ValueError('Disallowed subobject type: %s' % type_name)

        pt = getToolByName(self, 'portal_types')
        args = (type_name, self, id, RESPONSE) + args
        return pt.constructContent(*args, **kw)

    security.declareProtected(View, 'getDefaultAddableTypes')
    def getDefaultAddableTypes(self, context=None):
        """returns a list of normally allowed objects as ftis.
        Exactly like PortalFolder.allowedContentTypes except this
        will check in a specific context.
        """
        if context is None:
            context = self

        portal_types = getToolByName(self, 'portal_types')
        myType = portal_types.getTypeInfo(self)
        result = portal_types.listTypeInfo()
        # Don't give parameter context to portal_types.listTypeInfo().
        # If we do that, listTypeInfo will execute
        # t.isConstructionAllowed(context) for each content type
        # in portal_types.
        # The isConstructionAllowed should be done only on allowed types.
        if myType is not None:
            return [t for t in result if myType.allowType(t.getId()) and
                    t.isConstructionAllowed(context)]

        return [t for t in result if t.isConstructionAllowed(context)]

    security.declarePublic('canSetConstrainTypes')
    def canSetConstrainTypes(self):
        """Find out if the current user is allowed to set the allowable types
        """
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        return member.has_permission(ATCTPermissions.ModifyConstrainTypes, self)

    #
    # Helper methods
    #

    # Vocab for type lists
    security.declarePrivate('_ct_vocabularyPossibleTypes')
    def _ct_vocabularyPossibleTypes(self):
        """Get a DisplayList of types which may be added (id -> title)
        """
        typelist = [(fti.title_or_id(), fti.getId())
                     for fti in self.getDefaultAddableTypes()]
        typelist.sort()
        return DisplayList([(id, title) for title, id in typelist])

    # Default method for type lists
    security.declarePrivate('_ct_defaultAddableTypeIds')
    def _ct_defaultAddableTypeIds(self):
        """Get a list of types which are addable in the ordinary case w/o the
        constraint machinery.
        """
        return [fti.getId() for fti in self.getDefaultAddableTypes()]

    def _ct_defaultConstrainTypesMode(self):
        """Configure constrainTypeMode depending on the parent

        ACQUIRE if parent support ISelectableConstrainTypes
        DISABLE if not
        """
        portal_factory = getToolByName(self, 'portal_factory', None)
        if portal_factory is not None and portal_factory.isTemporary(self):
            # created by portal_factory
            parent = aq_parent(aq_parent(aq_parent(aq_inner(self))))
        else:
            parent = aq_parent(aq_inner(self))

        if ISelectableConstrainTypes.providedBy(parent) and parentPortalTypeEqual(self):
            return ACQUIRE
        else:
            return DISABLED

InitializeClass(ConstrainTypesMixin)



import logging

from Products.ATContentTypes.content import link

logger = logging.getLogger('ATContentTypes')
logger.log(logging.INFO, 'Warning: You have content instances of the '
    'ATFavorite type in your site. This type is no longer available. Please '
    'remove all remaining instances of this type or migrate them to links.')


class ATFavorite(link.ATLink):
    """BBB: The old and long removed favorite content type."""

    portal_type = 'Favorite'
    archetype_name = 'Favorite'



from zope.interface import implements

from DateTime import DateTime
from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import IntegerField
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import IntDisplayList

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import DATE_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics

from Products.ATContentTypes import ATCTMessageFactory as _

DateOptions = IntDisplayList((
    (0, _(u'Now')),
    (1, _(u'1 Day')),
    (2, _(u'2 Days')),
    (5, _(u'5 Days')),
    (7, _(u'1 Week')),
    (14, _(u'2 Weeks')),
    (31, _(u'1 Month')),
    (31 * 3, _(u'3 Months')),
    (31 * 6, _(u'6 Months')),
    (365, _(u'1 Year')),
    (365 * 2, _(u'2 Years')),
    ))

CompareOperations = DisplayList((
    ('more', _(u'More than')),
    ('less', _(u'Less than')),
    ('within_day', _(u'On the day')),
    ))

RangeOperations = DisplayList((
    ('-', _(u'in the past')),
    ('+', _(u'in the future')),
    ))

ATDateCriteriaSchema = ATBaseCriterionSchema + Schema((
    IntegerField('value',
                required=1,
                mode="rw",
                accessor="Value",
                mutator="setValue",
                write_permission=ChangeTopics,
                default=None,
                vocabulary=DateOptions,
                widget=SelectionWidget(
                    label=_(u'label_date_criteria_value', default=u'Which day'),
                    description=_(u'help_date_criteria_value',
                                  default=u'Select the date criteria value.')
                    ),
                ),
    StringField('dateRange',
                required=1,
                mode="rw",
                write_permission=ChangeTopics,
                default=None,
                vocabulary=RangeOperations,
                enforceVocabulary=1,
                widget=SelectionWidget(
                    label=_(u'label_date_criteria_range',
                            default=u'In the past or future'),
                    description=_(u'help_date_criteria_range',
                                  default=u"Select the date criteria range. Ignore this if you selected 'Now' above."),
                    format="select"),
                ),
    StringField('operation',
                required=1,
                mode="rw",
                default=None,
                write_permission=ChangeTopics,
                vocabulary=CompareOperations,
                enforceVocabulary=1,
                widget=SelectionWidget(
                    label=_(u'label_date_criteria_operation', default=u'More or less'),
                    description=_(u'help_date_criteria_operation',
                                  default=u'Select the date criteria operation.'),
                    format="select"),
                ),
    ))


class ATDateCriteria(ATBaseCriterion):
    """A relative date criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATDateCriteriaSchema
    meta_type = 'ATFriendlyDateCriteria'
    archetype_name = 'Friendly Date Criteria'
    shortDesc = 'Relative date'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        """Return a sequence of items to be used to build the catalog query.
        """
        if self.value is not None:
            field = self.Field()
            value = self.Value()

            # Negate the value for 'old' days
            if self.getDateRange() == '-':
                value = -value

            date = DateTime() + value
            current_date = DateTime()

            operation = self.getOperation()
            if operation == 'within_day':
                date_range = (date.earliestTime(), date.latestTime())
                return ((field, {'query': date_range, 'range': 'min:max'}),)
            elif operation == 'more':
                if value != 0:
                    range_op = (self.getDateRange() == '-' and 'max') or 'min'
                    return ((field, {'query': date.earliestTime(), 'range': range_op}),)
                else:
                    return ((field, {'query': date, 'range': 'min'}),)
            elif operation == 'less':
                if value != 0:
                    date_range = (self.getDateRange() == '-' and
                                  (date.earliestTime(), current_date)
                                  ) or (current_date, date.latestTime())
                    return ((field, {'query': date_range, 'range': 'min:max'}),)
                else:
                    return ((field, {'query': date, 'range': 'max'}),)
        else:
            return ()

registerCriterion(ATDateCriteria, DATE_INDICES)



## Script (Python) "formatHistoryDiff"
##title=Formats the history diff
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=lines

lst = []

map = {
    '+': '<ins>%s</ins><br />',
    '-': '<del>%s</del><br />',
    ' ': '&nbsp;%s<br />',
    '@': '<strong>%s</strong><br />',
}

for line in lines:
    first = len(line) and line[0] or ''
    mapping = map.get(first, '%s')
    print mapping % line

return printed



# -*- coding: utf-8 -*-

import unittest
from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase, atctftestcase

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.Archetypes.interfaces.layer import ILayerContainer
from Products.Archetypes import atapi
from Products.ATContentTypes.tests.utils import dcEdit
import StringIO

from plone.app.blob.content import ATBlob
from Products.ATContentTypes.interfaces import IATFile
from Products.ATContentTypes.interfaces import IFileContent
from zope.interface.verify import verifyObject


file_text = """
foooooo
"""


def editATCT(obj):
    dcEdit(obj)
    obj.edit(file=file_text)


class Fakefile(StringIO.StringIO):
    pass


tests = []


class TestSiteATFile(atcttestcase.ATCTTypeTestCase):

    klass = ATBlob
    portal_type = 'File'
    title = 'File'
    meta_type = 'ATFile'

    def test_implementsFileContent(self):
        iface = IFileContent
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_implementsATFile(self):
        iface = IATFile
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_edit(self):
        new = self._ATCT
        editATCT(new)

    def testCompatibilityFileAccess(self):
        new = self._ATCT
        editATCT(new)
        # test for crappy access ways of CMF :)
        self.assertEqual(str(new), file_text)
        self.assertEqual(new.data, file_text)
        self.assertEqual(str(new.getFile()), file_text)
        self.assertEqual(new.getFile().data, file_text)
        self.assertEqual(new.get_data(), file_text)

    def testCompatibilityContentTypeAccess(self):
        new = self._ATCT
        editATCT(new)
        # TODO: more tests

    def test_schema_marshall(self):
        atct = self._ATCT
        schema = atct.Schema()
        marshall = schema.getLayerImpl('marshall')
        marshallers = [atapi.PrimaryFieldMarshaller]
        try:
            from Products.Marshall import ControlledMarshaller
            marshallers.append(ControlledMarshaller)
        except ImportError:
            pass
        self.assertTrue(isinstance(marshall, tuple(marshallers)), marshall)

    def testInvokeFactoryWithFileContents(self):
        # test for Plone tracker #4939
        fakefile = Fakefile()
        fakefile.filename = 'some-filename'
        id = self.folder.invokeFactory(self.portal_type,
                                       'image.2005-11-18.4066860572',
                                       file=fakefile)
        self.assertEqual(id, fakefile.filename)

    def testUpperCaseFilename(self):
        fakefile = Fakefile()
        fakefile.filename = 'Some-filename-With-Uppercase.txt'
        id = 'file.2005-11-18.4066860573'
        self.folder.invokeFactory(self.portal_type, id)
        self.folder[id].setFile(fakefile)
        self.assertFalse(id in self.folder)
        self.assertTrue(fakefile.filename in self.folder)

    def testUpperCaseFilenameWithFunnyCharacters(self):
        fakefile = Fakefile()
        fakefile.filename = 'Zope&Plo?ne .txt'
        id = 'file.2005-11-18.4066860574'
        self.folder.invokeFactory(self.portal_type, id)
        self.folder[id].setFile(fakefile)
        self.assertFalse(id in self.folder)
        self.assertTrue('Zope-Plo-ne .txt' in self.folder)

    def testWindowsUploadFilename(self):
        fakefile = Fakefile()
        fakefile.filename = 'c:\\Windows\\Is\\Worthless\\file.txt'
        id = 'file.2005-11-18.4066860574'
        self.folder.invokeFactory(self.portal_type, id)
        self.folder[id].setFile(fakefile)
        self.assertFalse(id in self.folder)
        self.assertFalse(fakefile.filename in self.folder)
        self.assertTrue('file.txt' in self.folder)

    def testWindowsDuplicateFiles(self):
        fakefile = Fakefile()
        fakefile.filename = 'c:\\Windows\\Is\\Worthless\\file.txt'
        id = 'file.2005-11-18.4066860574'
        self.folder.invokeFactory(self.portal_type, id)
        self.folder[id].setFile(fakefile)
        self.folder.invokeFactory(self.portal_type, id)
        request = self.folder.REQUEST
        request.form['id'] = id
        request.form['file_file'] = fakefile
        errors = {}
        self.folder[id].post_validate(request, errors)
        self.assertTrue('file' in errors)

tests.append(TestSiteATFile)


class TestATFileFields(atcttestcase.ATCTFieldTestCase):

    # Title is not a required field, since we don't require them
    # on File/Image - they are taken from the filename if not present.
    # "Add the comment 'damn stupid fucking test'" -- optilude ;)
    def test_title(self):
        pass

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATBlob, subtype='File')

    def test_fileField(self):
        dummy = self._dummy
        field = dummy.getField('file')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 1, 'Value is %s' % field.required)
        self.assertFalse(field.default, 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == True, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == (),
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getFile',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setFile',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ModifyPortalContent,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'blob', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AnnotationStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AnnotationStorage(migrate=True),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.validators == "(('isNonEmptyFile', V_REQUIRED), ('checkFileMaxSize', V_REQUIRED))",
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.FileWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == (), 'Value is %s' % str(tuple(vocab)))
        self.assertTrue(field.primary == 1, 'Value is %s' % field.primary)

tests.append(TestATFileFields)


class TestCleanupFilename(atcttestcase.ATCTSiteTestCase):

    def test_cleanup_filename(self):
        self.app.REQUEST.set('HTTP_ACCEPT_LANGUAGE', 'el')
        from Products.ATContentTypes.content.base import cleanupFilename
        text = unicode('Νίκος Τζάνος', 'utf-8')
        self.assertEqual(cleanupFilename(text, request=self.app.REQUEST),
                          'Nikos Tzanos')

tests.append(TestCleanupFilename)


class TestATFileFunctional(atctftestcase.ATCTIntegrationTestCase):

    portal_type = 'File'
    views = ('file_view', 'download', )

    def test_inlineMimetypes_Office(self):
        # Only PDF and Office docs are shown inline
        self.obj.setFormat('application/msword')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), None)

        self.obj.setFormat('application/x-msexcel')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), None)

        self.obj.setFormat('application/vnd.ms-excel')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), None)

        self.obj.setFormat('application/vnd.ms-powerpoint')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), None)

    def test_inlineMimetypes_PDF(self):
        # Only PDF and Office docs are shown inline
        self.obj.setFormat('application/pdf')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), None)

    def test_inlineMimetypes_Text(self):
        # Only PDF and Office docs are shown inline
        self.obj.setFilename('foo.txt')
        self.obj.setFormat('text/plain')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), 'attachment; filename="foo.txt"')

    def test_inlineMimetypes_Binary(self):
        # Only PDF and Office docs are shown inline
        self.obj.setFilename('foo.exe')
        self.obj.setFormat('application/octet-stream')
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Disposition'), 'attachment; filename="foo.exe"')

tests.append(TestATFileFunctional)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



import datetime
from DateTime import DateTime


def dt2DT(date):
    """Convert Python's datetime to Zope's DateTime
    """
    return DateTime(date)


def DT2dt(date):
    """Convert Zope's DateTime to Pythons's datetime
    """
    return date.asdatetime()


def toTime(date):
    """get time part of a date
    """
    if isinstance(date, datetime.datetime):
        date = DateTime(date)
    return date.Time()


def toSeconds(td):
    """Converts a timedelta to an integer representing the number of seconds
    """
    return td.seconds + td.days * 86400



import json
from zope.i18n import translate
from zope.publisher.browser import BrowserView


class JSONCriteriaForField(BrowserView):
    """Return the criteria vocabulary as a json result"""

    def __call__(self):
        vocab = self.context.allowedCriteriaForField(self.request['field'],
            display_list=True)

        self.request.response.setHeader(
            'Content-Type', 'application/json; charset=utf-8')
        return json.dumps([
            dict(
                value=item,
                label=translate(vocab.getValue(item), domain="plone", context=self.request)
            ) for item in vocab
        ])



"""AT Content Types configuration file

DO NOT CHANGE THIS FILE!

Use ZConfig to configure ATCT
"""
__docformat__ = 'restructuredtext'

import pkg_resources
import os
from Products.ATContentTypes.configuration import zconf

## options for mx tidy
## read http://www.egenix.com/files/python/mxTidy.html for more informations
MX_TIDY_ENABLED = zconf.mxtidy.enable
MX_TIDY_OPTIONS = zconf.mxtidy.options

###############################################################################
## private options

PROJECTNAME = "ATContentTypes"
TOOLNAME = "portal_atct"
SKINS_DIR = 'skins'

ATCT_DIR = os.path.abspath(os.path.dirname(__file__))
WWW_DIR = os.path.join(ATCT_DIR, 'www')

GLOBALS = globals()

## swallow PIL exceptions when resizing the image?
SWALLOW_IMAGE_RESIZE_EXCEPTIONS = zconf.swallowImageResizeExceptions.enable

## mxTidy available?
try:
    # I am not sure, which pkg should contain mx.Tidy
    from mx import Tidy
except ImportError:
    HAS_MX_TIDY = False
else:
    HAS_MX_TIDY = True
    try:
        del Tidy
    except AttributeError:
        pass

## tidy only these document types
MX_TIDY_MIMETYPES = (
    'text/html',
     )

## ExternalStorage available?
try:
    pkg_resources.get_distribution('Products.ExternalStorage')
except pkg_resources.DistributionNotFound:
    HAS_EXT_STORAGE = False
else:
    HAS_EXT_STORAGE = True

## LinguaPlone addon?
try:
    pkg_resources.get_distribution('Products.LinguaPlone')
except pkg_resources.DistributionNotFound:
    HAS_LINGUA_PLONE = False
else:
    HAS_LINGUA_PLONE = True

try:
    # Won't use pkg_resources because of the packaging issue
    from PIL import Image
except ImportError:
    HAS_PIL = False
else:
    HAS_PIL = True


## workflow mapping for the installer
WORKFLOW_DEFAULT = '(Default)'
WORKFLOW_FOLDER = 'folder_workflow'
WORKFLOW_TOPIC = 'folder_workflow'
WORKFLOW_CRITERIA = ''

## icon map used for overwriting ATFile icons
ICONMAP = {'application/pdf': 'pdf_icon.png',
           'image': 'image_icon.png'}

MIME_ALIAS = {
    'plain': 'text/plain',
    'stx': 'text/structured',
    'html': 'text/html',
    'rest': 'text/x-rst',
    'text/stx': 'text/structured',
    'structured-text': 'text/structured',
    'restructuredtext': 'text/x-rst',
    'text/restructured': 'text/x-rst',
    }



"""AT Content Types general interfaces

BBB: We used to have all interfaces specified in "interface". "interfaces" is
the conventional name, though.
"""

from Products.ATContentTypes.interfaces import *

# the following is a rather crude workaround for the failing imports
# seen in plone 4 when trying to import submodules from `ATCT.interface`:
#   >>> import Products.ATContentTypes.interface.interfaces
#   Traceback (most recent call last):
#     File "<stdin>", line 1, in <module>
#   ImportError: No module named interfaces
# apparently the modules imported from `interfaces` above are already
# somehow known to the interpreter and therefore not added to `sys.modules`
# again.  to work around we inject them manually...
from types import ModuleType
from sys import modules
for name, obj in globals().items():
    if type(obj) is ModuleType:
        modules['%s.%s' % (__name__, name)] = obj



import os

from ZConfig.loader import ConfigLoader
from Globals import INSTANCE_HOME
from Products.ATContentTypes.configuration.schema import atctSchema

# directories
INSTANCE_ETC = os.path.join(INSTANCE_HOME, 'etc')
_here = os.path.dirname(__file__)
ATCT_HOME = os.path.dirname(os.path.abspath(os.path.join(_here)))
ATCT_ETC = os.path.join(ATCT_HOME, 'etc')

# files
CONFIG_NAME = 'atcontenttypes.conf'
INSTANCE_CONFIG = os.path.join(INSTANCE_ETC, CONFIG_NAME)
ATCT_CONFIG = os.path.join(ATCT_ETC, CONFIG_NAME)
ATCT_CONFIG_IN = os.path.join(ATCT_ETC, CONFIG_NAME + '.in')

# check files for existence
if not os.path.isfile(INSTANCE_CONFIG):
    INSTANCE_CONFIG = None
if not os.path.isfile(ATCT_CONFIG):
    ATCT_CONFIG = None
if not os.path.isfile(ATCT_CONFIG_IN):
    raise RuntimeError("Unable to find configuration file at %s" %
                        ATCT_CONFIG_IN)
FILES = (INSTANCE_CONFIG, ATCT_CONFIG, ATCT_CONFIG_IN,)

# config
zconf, handler, conf_file = None, None, None


def loadConfig(files, schema=atctSchema, overwrite=False):
    """Config loader

    The config loader tries to load the first existing file
    """
    global zconf, handler, conf_file
    if not isinstance(files, (tuple, list)):
        files = (files, )
    if zconf is not None and not overwrite:
        raise RuntimeError, 'Configuration is already loaded'
    for file in files:
        if file is not None:
            if not os.path.exists(file):
                raise RuntimeError, '%s does not exist' % file
            conf_file = file
            zconf, handler = ConfigLoader(schema).loadURL(conf_file)
            break


loadConfig(FILES)

__all__ = ('zconf', 'handler', 'conf_file')



from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.base import ATCTFileContent
from Products.ATContentTypes.content.base import ATCTFolder
from Products.ATContentTypes.content.base import ATCTOrderedFolder
from Products.ATContentTypes.content.base import ATCTBTreeFolder

from Products.ATContentTypes.content.schemata import ATContentTypeSchema

from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.event import ATEvent
from Products.ATContentTypes.content.file import ATFile
from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.image import ATImage
from Products.ATContentTypes.content.link import ATLink
from Products.ATContentTypes.content.newsitem import ATNewsItem
from Products.ATContentTypes.content.topic import ATTopic

from Products.ATContentTypes.content.document import ATDocumentSchema
from Products.ATContentTypes.content.event import ATEventSchema
from Products.ATContentTypes.content.file import ATFileSchema
from Products.ATContentTypes.content.folder import ATFolderSchema
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ATContentTypes.content.image import ATImageSchema
from Products.ATContentTypes.content.link import ATLinkSchema
from Products.ATContentTypes.content.newsitem import ATNewsItemSchema
from Products.ATContentTypes.content.topic import ATTopicSchema

from Products.ATContentTypes.criteria import ALL_INDICES
from Products.ATContentTypes.criteria import DATE_INDICES
from Products.ATContentTypes.criteria import STRING_INDICES
from Products.ATContentTypes.criteria import LIST_INDICES
from Products.ATContentTypes.criteria import SORT_INDICES
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import unregisterCriterion

from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema

from Products.ATContentTypes.criteria.boolean import ATBooleanCriterion
from Products.ATContentTypes.criteria.date import ATDateCriteria
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterion
from Products.ATContentTypes.criteria.list import ATListCriterion
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterion
from Products.ATContentTypes.criteria.reference import ATReferenceCriterion
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterion
from Products.ATContentTypes.criteria.simplestring import ATSimpleStringCriterion
from Products.ATContentTypes.criteria.sort import ATSortCriterion

from Products.ATContentTypes.criteria.boolean import ATBooleanCriterionSchema
from Products.ATContentTypes.criteria.date import ATDateCriteriaSchema
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterionSchema
from Products.ATContentTypes.criteria.list import ATListCriterionSchema
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterionSchema
from Products.ATContentTypes.criteria.reference import ATReferenceCriterionSchema
from Products.ATContentTypes.criteria.selection import ATSelectionCriterionSchema
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterionSchema
from Products.ATContentTypes.criteria.simplestring import ATSimpleStringCriterionSchema
from Products.ATContentTypes.criteria.sort import ATSortCriterionSchema



from types import FileType
from Acquisition import aq_base

from Products.ATContentTypes.config import HAS_MX_TIDY
from Products.ATContentTypes.config import MX_TIDY_ENABLED
from Products.ATContentTypes.config import MX_TIDY_MIMETYPES
from Products.ATContentTypes.config import MX_TIDY_OPTIONS

from Products.validation.config import validation
from Products.validation.interfaces.IValidator import IValidator

import re
import encodings
import logging
logger = logging.getLogger('ATCT')

from ZPublisher.HTTPRequest import FileUpload

from zope.tal.htmltalparser import HTMLTALParser
from zope.tal.talgenerator import TALGenerator
from Products.PageTemplates.Expressions import getEngine
from zope.interface import implements

if HAS_MX_TIDY:
    from mx.Tidy import tidy as mx_tidy

# matches something like 'line 15 column 1 - Warning: missing ...'
RE_MATCH_WARNING = re.compile('^line (\d+) column (\d+) - Warning: (.*)$')
WARNING_LINE = 'line %d column %d - Warning: %s'

# matches something like 'line 15 column 1 - Error: missing ...'
RE_MATCH_ERROR = re.compile('^line (\d+) column (\d+) - Error: (.*)$')
ERROR_LINE = 'line %d column %d - Error: %s'

# the following regex is safe because *? matches the minimal text in the body tag
# and .* matches the maximum text between two body tags including other body tags
# if they exists
RE_BODY = re.compile('<body[^>]*?>(.*)</body>', re.DOTALL)

# get the encoding from an uploaded html-page
# e.g. <meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
# we get ISO-8859-1 into the second match, the rest into the first and third.
RE_GET_HTML_ENCODING = re.compile('(<meta.*?content-type.*?charset[\s]*=[\s]*)([^"]*?)("[^>]*?>)', re.S | re.I)

# subtract 11 line numbers from the warning/error
SUBTRACT_LINES = 11

validatorList = []


class TALValidator:
    """Validates a text to be valid TAL code

    """
    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        gen = TALGenerator(getEngine(), xml=1, source_file=None)
        parser = HTMLTALParser(gen)
        try:
            parser.parseString(value)
        except Exception, err:
            return ("Validation Failed(%s): \n %s" % (self.name, err))
        return 1

validatorList.append(TALValidator('isTAL', title='', description=''))


class TidyHtmlValidator:
    """use mxTidy to check HTML

    Fail on errors and warnings
    Do not clean up the value
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        if not (HAS_MX_TIDY and MX_TIDY_ENABLED):
            # no mxTidy installed
            return 1

        request = kw['REQUEST']
        field = kw['field']

        result = doTidy(value, field, request)
        if result is None:
            return 1

        nerrors, nwarnings, outputdata, errordata = result
        errors = nerrors + nwarnings

        if errors:
            return ("Validation Failed(%s): \n %s" % (self.name, errordata))
        else:
            return 1

validatorList.append(TidyHtmlValidator('isTidyHtml', title='', description=''))


class TidyHtmlWithCleanupValidator:
    """use mxTidy to check HTML

    Fail only on errors
    Clean up
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        if not (HAS_MX_TIDY and MX_TIDY_ENABLED):
            # no mxTidy installed
            return 1

        request = kw['REQUEST']
        field = kw['field']

        result = doTidy(value, field, request, cleanup=1)
        if result is None:
            return 1

        nerrors, nwarnings, outputdata, errordata = result
        errors = nerrors

        # save the changed output in the request
        tidyAttribute = '%s_tidier_data' % field.getName()
        request[tidyAttribute] = outputdata

        if nwarnings:
            tidiedFields = list(request.get('tidiedFields', []))
            tidiedFields.append(field)
            request.set('tidiedFields', tidiedFields)

        if errors:
            return ("Validation Failed(%s): \n %s" % (self.name, errordata))
        else:
            return 1

validatorList.append(TidyHtmlWithCleanupValidator('isTidyHtmlWithCleanup', title='', description=''))


class NonEmptyFileValidator:
    """Fails on empty non-existant files
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        instance = kwargs.get('instance', None)
        field = kwargs.get('field', None)

        # calculate size
        if isinstance(value, FileUpload) or type(value) is FileType \
          or hasattr(aq_base(value), 'tell'):
            value.seek(0, 2)  # eof
            size = value.tell()
            value.seek(0)
        else:
            try:
                size = len(value)
            except TypeError:
                size = 1

        if size == 0:
            return ("Validation failed: Uploaded file is empty")
        else:
            return True


validatorList.append(NonEmptyFileValidator('isNonEmptyFile', title='', description=''))

for validator in validatorList:
    # register the validators
    validation.register(validator)


def doTidy(value, field, request, cleanup=0):
    """Tidy the data in 'value' for the field in the current request

    Optional cleanup:
      * removes header/footer of the output data
      * Removes warnings from the error data

    Return None for 'nothing done'
    else return (nerrors, nwarnings, outputdata, errordata)
    """
    # we can't use the mimetype from the field because it's updated *after*
    # validation so we must get it from the request
    tf_name = '%s_text_format' % field.getName()
    text_format = getattr(request, tf_name, '')

    # MX_TIDY_MIMETYPES configuration option isn't empty
    # and the current text_format isn't in the list
    if MX_TIDY_MIMETYPES and text_format not in MX_TIDY_MIMETYPES:
        # do not filter this mime type
        return

    # it's a file upload
    if isinstance(value, FileUpload):
        # *mmh* ok it's a file upload but a file upload could destroy
        # the layout, too.
        # the validator can be called many times, we have to rewind
        # the FileUpload.
        value.seek(0)
        value = correctEncoding(value.read())
    else:
        value = wrapValueInHTML(value)

    result = mx_tidy(value, **MX_TIDY_OPTIONS)
    nerrors, nwarnings, outputdata, errordata = result

    # parse and change the error data
    errordata = parseErrorData(errordata, removeWarnings=cleanup)
    if cleanup and outputdata:
        # unwrap tidied output data
        outputdata = unwrapValueFromHTML(outputdata)

    return nerrors, nwarnings, outputdata, errordata


def wrapValueInHTML(value):
    """Wrap the data in a valid html construct to remove the missing title error
    """
    return """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title></title>
</head>
<body>
%s
</body>
</html>
""" % value


def unwrapValueFromHTML(value):
    """Remove the html stuff around the body
    """
    # get the body text
    result = RE_BODY.search(value)
    if result:
        body = result.group(1)
    else:
        raise ValueError('%s is not a html string' % value)

##    # remove 2 spaces from the beginning of each line
##    nlines = []
##    for line in body.split('\n'):
##        print line
##        if line[:2] == '  ':
##            nlines.append(line[2:])
##        else:
##            nlines.append(line)
##
##    return '\n'.join(nlines)
    return body


def correctEncoding(value):
    """correct the encoding of a html-page if we know it an mxTidy
       expects an other encoding
    """

    # we have nothing to do if mxTidy has no
    # fixed char_encoding
    if 'char_encoding' not in MX_TIDY_OPTIONS  \
           or (MX_TIDY_OPTIONS['char_encoding'] == 'raw'):
        return value

    match = RE_GET_HTML_ENCODING.search(value)
    if match:
        groups = match.groups()

        # lookup encodings in the pyhon encodings database
        # returns function-pointers that we can compare
        # need to normalize encodings a bit before
        html_encoding = groups[1].strip().lower()
        char_encoding = MX_TIDY_OPTIONS['char_encoding'].lower().strip()
        h_enc = encodings.search_function(html_encoding)
        c_enc = encodings.search_function(char_encoding)

        # one encoding is missing or they are equal
        if not (h_enc and c_enc) or  h_enc == c_enc:
            return value
        else:
            try:
                return unicode(value, html_encoding).encode(char_encoding)
            except:
                logger.info("Error correcting encoding from %s to %s" % (html_encoding, char_encoding))
    return value


def parseErrorData(data, removeWarnings=0):
    """Parse the error data to change some stuff
    """
    lines = data.split('\n')
    nlines = []
    for line in lines:
        # substract 11 lines from line
        error = RE_MATCH_ERROR.search(line)
        if error:
            # an error line
            lnum, cnum, text = error.groups()
            lnum = int(lnum) - SUBTRACT_LINES
            cnum = int(cnum)
            nlines.append(ERROR_LINE % (lnum, cnum, text))
        else:
            warning = RE_MATCH_WARNING.search(line)
            if warning and not removeWarnings:
                # a warning line and add warnings to output
                lnum, cnum, text = warning.groups()
                lnum = int(lnum) - SUBTRACT_LINES
                cnum = int(cnum)
                nlines.append(WARNING_LINE % (lnum, cnum, text))
            else:
                # something else
                nlines.append(line)
    return '\n'.join(nlines)



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase, atctftestcase

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.Archetypes.interfaces.layer import ILayerContainer
from Products.Archetypes import atapi
from Products.ATContentTypes.tests.utils import dcEdit

from Products.ATContentTypes.content.newsitem import ATNewsItem
from Products.ATContentTypes.tests.utils import NotRequiredTidyHTMLValidator
from Products.ATContentTypes.interfaces import ITextContent
from Products.ATContentTypes.interfaces import IImageContent
from Products.ATContentTypes.interfaces import IATNewsItem
from zope.interface.verify import verifyObject


tests = []

TEXT = "lorum ipsum"


def editATCT(obj):
    dcEdit(obj)
    obj.setText(TEXT)


class TestSiteATNewsItem(atcttestcase.ATCTTypeTestCase):

    klass = ATNewsItem
    portal_type = 'News Item'
    title = 'News Item'
    meta_type = 'ATNewsItem'

    def test_implementsTextContent(self):
        iface = ITextContent
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_implementsImageContent(self):
        iface = IImageContent
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_implementsATNewsItem(self):
        iface = IATNewsItem
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_edit(self):
        new = self._ATCT
        editATCT(new)

    def test_get_size(self):
        atct = self._ATCT
        editATCT(atct)
        self.assertEqual(atct.get_size(), len(TEXT))

tests.append(TestSiteATNewsItem)


class TestATNewsItemFields(atcttestcase.ATCTFieldTestCase):

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATNewsItem)

    def test_textField(self):
        dummy = self._dummy
        field = dummy.getField('text')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 0, 'Value is %s' % field.required)
        self.assertTrue(field.default == '', 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 1, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == (),
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getText',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setText',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ModifyPortalContent,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'text', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AnnotationStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AnnotationStorage(migrate=True),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.validators == NotRequiredTidyHTMLValidator,
                        'Value is %s' % repr(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.RichWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == (), 'Value is %s' % str(tuple(vocab)))

        self.assertTrue(field.primary == 1, 'Value is %s' % field.primary)
        self.assertTrue(field.default_content_type is None,
                        'Value is %s' % field.default_content_type)
        self.assertTrue(field.default_output_type == 'text/x-html-safe',
                        'Value is %s' % field.default_output_type)
        self.assertTrue('text/html' in field.getAllowedContentTypes(dummy))

tests.append(TestATNewsItemFields)


class TestATNewsItemFunctional(atctftestcase.ATCTIntegrationTestCase):

    portal_type = 'News Item'
    views = ('newsitem_view', )

tests.append(TestATNewsItemFunctional)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from zope.interface import implements

from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import REFERENCE_INDICES
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.Archetypes.atapi import DisplayList

ATReferenceCriterionSchema = ATSelectionCriterion.schema


class ATReferenceCriterion(ATSelectionCriterion):
    """A reference criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    meta_type = 'ATReferenceCriterion'
    archetype_name = 'Reference Criterion'
    shortDesc = 'Select referenced content'

    def getCurrentValues(self):
        catalog = getToolByName(self, 'portal_catalog')
        uid_cat = getToolByName(self, 'uid_catalog')
        putils = getToolByName(self, 'plone_utils')
        options = catalog.uniqueValuesFor(self.Field())

        # If the uid catalog has a Language index restrict the query by it.
        # We should only shows references to items in the same or the neutral
        # language.
        query = dict(UID=options, sort_on='Title')
        if 'Language' in uid_cat.indexes():
            query['Language'] = [self.Language(), '']

        brains = uid_cat(**query)
        display = [((putils.pretty_title_or_id(b)).lower(), b.UID, b.Title or b.id) for b in brains]
        display.sort()
        display_list = DisplayList([(d[1], d[2]) for d in display])

        return display_list

registerCriterion(ATReferenceCriterion, REFERENCE_INDICES)



from zope.interface import implements

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import LinesWidget
from Products.Archetypes.atapi import DisplayList

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import LIST_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics

from Products.ATContentTypes import ATCTMessageFactory as _

CompareOperators = DisplayList((
    ('and', _(u'and')),
    ('or', _(u'or')),
    ))

ATListCriterionSchema = ATBaseCriterionSchema + Schema((
    LinesField('value',
                required=1,
                mode="rw",
                write_permission=ChangeTopics,
                accessor="Value",
                mutator="setValue",
                default=[],
                widget=LinesWidget(
                    label=_(u'label_list_criteria_value', default=u'Values'),
                    description=_(u'help_list_criteria_value',
                                  default=u'Values, each on its own line.')
                    ),
                ),
    StringField('operator',
                required=1,
                mode="rw",
                write_permission=ChangeTopics,
                default='or',
                vocabulary=CompareOperators,
                widget=SelectionWidget(
                    label=_(u'label_list_criteria_operator',
                            default=u'operator name'),
                    description=_(u'help_list_criteria_operator',
                                  default=u'Operator used to join the tests on each value.')
                    ),
                ),
    ))


class ATListCriterion(ATBaseCriterion):
    """A list criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATListCriterionSchema
    meta_type = 'ATListCriterion'
    archetype_name = 'List Criterion'
    shortDesc = 'List of values'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        # filter out empty strings
        result = []

        value = tuple([value for value in self.Value() if value])
        if not value:
            return ()
        result.append((self.Field(), {'query': value, 'operator': self.getOperator()}),)

        return tuple(result)

registerCriterion(ATListCriterion, LIST_INDICES)



## Script (Python) "unittestGetTitleOf"
##title=Helper method for function tests
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=

return "%s,%s,%s,%s" % (context.title_or_id(), context.aq_parent.title_or_id(),\
 context.aq_inner.title_or_id(), context.aq_inner.aq_parent.title_or_id())



from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase

from Products.ATContentTypes.config import TOOLNAME
from Products.ATContentTypes.migration.v1_2 import upgradeATCTTool
from Products.CMFCore.utils import getToolByName


class TestMigrations_v1_2(atcttestcase.ATCTSiteTestCase):

    def afterSetUp(self):
        self.tool = getToolByName(self.portal, TOOLNAME)

    def testUpgradeATCTTool(self):
        self.assertEqual(self.tool.getProperty('album_batch_size'), 30)
        self.tool._setPropValue('album_batch_size', 99)
        self.tool._setPropValue('_version', '1.1.x (svn/testing)')
        upgradeATCTTool(self.portal)
        self.assertEqual(self.tool.getProperty('album_batch_size'), 99)

    def testUpgradeATCTToolTwice(self):
        self.assertEqual(self.tool.getProperty('album_batch_size'), 30)
        self.tool._setPropValue('album_batch_size', 99)
        self.tool._setPropValue('_version', '1.1.x (svn/testing)')
        upgradeATCTTool(self.portal)
        upgradeATCTTool(self.portal)
        self.assertEqual(self.tool.getProperty('album_batch_size'), 99)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMigrations_v1_2))
    return suite



import unittest

from DateTime import DateTime

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase
from Missing import MV

from zope.interface.verify import verifyObject
from Products.Archetypes.interfaces.base import IBaseContent
from Products.Archetypes.interfaces.referenceable import IReferenceable
from Products.Archetypes.interfaces import IExtensibleMetadata

from Products.ATContentTypes.interfaces import IATTopicCriterion

from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.date import ATDateCriteria
from Products.ATContentTypes.criteria.list import ATListCriterion
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterion
from Products.ATContentTypes.criteria.simplestring import \
    ATSimpleStringCriterion
from Products.ATContentTypes.criteria.sort import ATSortCriterion
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterion
from Products.ATContentTypes.criteria.reference import ATReferenceCriterion
from Products.ATContentTypes.criteria.boolean import ATBooleanCriterion
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterion
from Products.ATContentTypes.criteria.currentauthor import \
    ATCurrentAuthorCriterion
from Products.ATContentTypes.criteria.path import ATPathCriterion
from Products.ATContentTypes.criteria.relativepath import ATRelativePathCriterion
tests = []


class CriteriaTest(atcttestcase.ATCTSiteTestCase):

    klass = None
    portal_type = None
    title = None
    meta_type = None

    def afterSetUp(self):
        atcttestcase.ATCTSiteTestCase.afterSetUp(self)
        self.dummy = self.createDummy(self.klass)

    def createDummy(self, klass, id='dummy'):
        folder = self.folder
        if klass is not None:
            dummy = klass(id, 'dummyfield')
            # put dummy in context of portal
            folder._setObject(id, dummy)
            dummy = getattr(folder, id)
            dummy.initializeArchetype()
        else:
            dummy = None
        return dummy

    def test_000testsetup(self):
        if self.klass is not None:
            self.assertTrue(self.klass)
            self.assertTrue(self.portal_type)
            self.assertTrue(self.title)
            self.assertTrue(self.meta_type)

    def test_multipleCreateVariants(self):
        klass = self.klass
        id = 'dummy'
        field = 'dummyfield'
        if klass is not None:
            dummy = klass(id, field)
            self.assertTrue(dummy.getId(), id)
            self.assertTrue(dummy.Field(), field)

            dummy = klass(id=id, field=field)
            self.assertTrue(dummy.getId(), id)
            self.assertTrue(dummy.Field(), field)

            dummy = klass(field, oid=id)
            self.assertTrue(dummy.getId(), id)
            self.assertTrue(dummy.Field(), field)

            dummy = klass(field=field, oid=id)
            self.assertTrue(dummy.getId(), id)
            self.assertTrue(dummy.Field(), field)

    def test_typeInfo(self):
        if self.dummy is not None:
            ti = self.dummy.getTypeInfo()
            self.assertEqual(ti.getId(), self.portal_type)
            self.assertEqual(ti.Title(), self.title)
            self.assertEqual(ti.Metatype(), self.meta_type)

    def test_implements(self):
        if self.dummy is not None:
            self.assertFalse(IReferenceable.providedBy(self.dummy))
            self.assertFalse(IExtensibleMetadata.providedBy(self.dummy))
            self.assertFalse(self.dummy.isReferenceable)
            self.assertTrue(IBaseContent.providedBy(self.dummy))
            self.assertTrue(IATTopicCriterion.providedBy(self.dummy))
            self.assertTrue(verifyObject(IBaseContent, self.dummy))
            self.assertTrue(verifyObject(IATTopicCriterion, self.dummy))


class TestATBaseCriterion(CriteriaTest):
    klass = ATBaseCriterion
    title = 'Base Criterion'
    meta_type = 'ATBaseCriterion'
    portal_type = 'ATBaseCriterion'

    def test_typeInfo(self):
        # not registered
        pass

tests.append(TestATBaseCriterion)


class TestATDateCriteria(CriteriaTest):
    klass = ATDateCriteria
    title = 'Friendly Date Criteria'
    meta_type = 'ATFriendlyDateCriteria'
    portal_type = 'ATDateCriteria'

    def test_LessThanPast(self):
        # A query of the form 'Less than 14 days ago' should generate a min:max
        # query with a start 14 days in the past and an end in the present
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        self.dummy.setOperation('less')
        self.dummy.setDateRange('-')
        self.dummy.setValue('14')
        expected_begin = (DateTime() - 14).earliestTime()
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'created')
        #range should start in past at the beginning of the day
        self.assertEqual(query['query'][0], expected_begin)
        # range should end today
        self.assertEqual(query['query'][1].earliestTime(),
                                                    DateTime().earliestTime())
        self.assertEqual(query['range'], 'min:max')

    def test_LessThanFuture(self):
        # A query of the form 'Less than 14 days in the future' should generate
        # a min:max query with an end 14 days in the future and a start in the
        # present
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        self.dummy.setOperation('less')
        self.dummy.setDateRange('+')
        self.dummy.setValue('14')
        expected_end = (DateTime() + 14).latestTime()
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        #Range should end on future date at the end of the day
        self.assertEqual(query['query'][1], expected_end)
        #Range should start today
        self.assertEqual(query['query'][0].earliestTime(),
                                                    DateTime().earliestTime())
        self.assertEqual(query['range'], 'min:max')

    def test_MoreThanPast(self):
        # A query of the form 'More than 14 days ago' should generate a max
        # query with the value set to a date 14 days in the past.
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        self.dummy.setOperation('more')
        self.dummy.setDateRange('-')
        self.dummy.setValue('14')
        expected_begin = (DateTime() - 14).earliestTime()
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'], expected_begin)
        self.assertEqual(query['range'], 'max')

    def test_MoreThanFuture(self):
        # A query of the form 'More than 14 days from now' should generate
        # a min query with the value set to a date 14 days in the future.
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        self.dummy.setOperation('more')
        self.dummy.setDateRange('+')
        self.dummy.setValue('14')
        expected_begin = (DateTime() + 14).earliestTime()
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'], expected_begin)
        self.assertEqual(query['range'], 'min')

    def test_MoreThanNow(self):
        # A query of the form 'More than Now' should generate
        # a min query with the value set to the present, regardless of the
        # past future setting.
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        self.dummy.setOperation('more')
        self.dummy.setDateRange('+')
        self.dummy.setValue('0')
        expected_begin = DateTime().earliestTime()
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'].earliestTime(), expected_begin)
        self.assertEqual(query['range'], 'min')
        # Change past/future setting
        self.dummy.setDateRange('-')
        self.assertEqual(query['query'].earliestTime(), expected_begin)
        self.assertEqual(query['range'], 'min')

    def test_LessThanNow(self):
        # A query of the form 'Less than Now' should generate
        # a max query with the value set to the present, regardless of the
        # past future setting.
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        self.dummy.setOperation('less')
        self.dummy.setDateRange('+')
        self.dummy.setValue('0')
        expected_begin = DateTime().earliestTime()
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'].earliestTime(), expected_begin)
        self.assertEqual(query['range'], 'max')
        # Change past/future setting
        self.dummy.setDateRange('-')
        self.assertEqual(query['query'].earliestTime(), expected_begin)
        self.assertEqual(query['range'], 'max')

tests.append(TestATDateCriteria)


class TestATListCriterion(CriteriaTest):
    klass = ATListCriterion
    title = 'List Criterion'
    meta_type = 'ATListCriterion'
    portal_type = 'ATListCriterion'

    def test_list_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'Subject')
        self.dummy.setOperator('or')
        self.dummy.setValue(('1', '2', '3'))
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'Subject')
        self.assertEqual(query['query'], ('1', '2', '3'))
        self.assertEqual(query['operator'], 'or')

tests.append(TestATListCriterion)


class TestATSimpleIntCriterion(CriteriaTest):
    klass = ATSimpleIntCriterion
    title = 'Simple Int Criterion'
    meta_type = 'ATSimpleIntCriterion'
    portal_type = 'ATSimpleIntCriterion'

    def test_base_int_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getObjPositionInParent')
        self.dummy.setDirection('')
        self.dummy.setValue(12)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'getObjPositionInParent')
        self.assertEqual(query['query'], 12)

    def test_int_min(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getObjPositionInParent')
        self.dummy.setDirection('min')
        self.dummy.setValue(12)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'], 12)
        self.assertEqual(query['range'], 'min')

    def test_int_max(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getObjPositionInParent')
        self.dummy.setDirection('max')
        self.dummy.setValue(12)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'], 12)
        self.assertEqual(query['range'], 'max')

    def test_int_between(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getObjPositionInParent')
        self.dummy.setDirection('min:max')
        self.dummy.setValue(12)
        self.dummy.setValue2(17)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        self.assertEqual(query['query'], (12, 17))
        self.assertEqual(query['range'], 'min:max')

tests.append(TestATSimpleIntCriterion)


class TestATSimpleStringCriterion(CriteriaTest):
    klass = ATSimpleStringCriterion
    title = 'Simple String Criterion'
    meta_type = 'ATSimpleStringCriterion'
    portal_type = 'ATSimpleStringCriterion'

    def test_string_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'Subject')
        self.dummy.setValue('a*')
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'Subject')
        self.assertEqual(query, 'a*')

tests.append(TestATSimpleStringCriterion)


class TestATSortCriterion(CriteriaTest):
    klass = ATSortCriterion
    title = 'Sort Criterion'
    meta_type = 'ATSortCriterion'
    portal_type = 'ATSortCriterion'

    def test_sort_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getObjPositionInParent')
        self.dummy.setReversed(False)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        self.assertEqual(items[0][0], 'sort_on')
        self.assertEqual(items[0][1], 'getObjPositionInParent')

    def test_list_query_reversed(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getObjPositionInParent')
        self.dummy.setReversed(True)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 2)
        self.assertEqual(items[0][0], 'sort_on')
        self.assertEqual(items[0][1], 'getObjPositionInParent')
        self.assertEqual(items[1][0], 'sort_order')
        self.assertEqual(items[1][1], 'reverse')

tests.append(TestATSortCriterion)


class TestATSelectionCriterion(CriteriaTest):
    klass = ATSelectionCriterion
    title = 'Selection Criterion'
    meta_type = 'ATSelectionCriterion'
    portal_type = 'ATSelectionCriterion'

    #Same as list criterion but without operator and with special vocabulary
    def test_selection_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'Subject')
        self.dummy.setValue(('1', '2', '3'))
        self.dummy.setOperator('and')
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'Subject')
        self.assertEqual(query['query'], ('1', '2', '3'))
        self.assertEqual(query['operator'], 'and')

    def test_vocabulary(self):
        #Should return some ids
        self.dummy.Schema()['field'].set(self.dummy, 'getId')
        self.assertTrue(self.dummy.getCurrentValues())

    def test_vocabulary_sorted(self):
        #Should return sorted ids
        self.dummy.Schema()['field'].set(self.dummy, 'getId')
        orig_vocab = [a.lower() for a in list(self.dummy.getCurrentValues())]
        sorted_vocab = orig_vocab[:]
        sorted_vocab.sort()
        self.assertEqual(orig_vocab, sorted_vocab)

tests.append(TestATSelectionCriterion)


class TestATDateRangeCriterion(CriteriaTest):
    klass = ATDateRangeCriterion
    title = 'Date Range Criterion'
    meta_type = 'ATDateRangeCriterion'
    portal_type = 'ATDateRangeCriterion'

    def test_date_range_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'created')
        now = DateTime()
        self.dummy.setStart(now)
        self.dummy.setEnd(now + 5)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'created')
        self.assertEqual(query['query'][0], now)
        self.assertEqual(query['query'][1], now + 5)
        self.assertEqual(query['range'], 'min:max')

tests.append(TestATDateRangeCriterion)


class TestATReferenceCriterion(CriteriaTest):
    klass = ATReferenceCriterion
    title = 'Reference Criterion'
    meta_type = 'ATReferenceCriterion'
    portal_type = 'ATReferenceCriterion'

    #Same as list criterion but without operator and with special vocabulary
    def test_reference_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getRawRelatedItems')
        self.folder.invokeFactory('Document', 'doc1')
        uid = self.folder.doc1.UID()
        self.dummy.setValue((uid,))
        self.dummy.setOperator('and')
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'getRawRelatedItems')
        self.assertEqual(query['query'], (uid,))
        self.assertEqual(query['operator'], 'and')

    def test_reference_vocab(self):
        self.dummy.Schema()['field'].set(self.dummy, 'getRawRelatedItems')
        self.folder.invokeFactory('Document', 'doc1')
        self.folder.invokeFactory('Document', 'doc2')
        uid1 = self.folder.doc1.UID()
        uid2 = self.folder.doc2.UID()
        self.folder.doc1.setRelatedItems([uid2])
        self.folder.doc2.setRelatedItems([uid1])
        self.folder.doc1.reindexObject()
        self.folder.doc2.reindexObject()
        vocab = self.dummy.getCurrentValues()
        self.assertEqual(len(vocab), 2)
        self.assertTrue(uid1 in vocab.keys())
        self.assertTrue(uid2 in vocab.keys())
        self.assertTrue('doc1' in vocab.values())
        self.assertTrue('doc2' in vocab.values())

tests.append(TestATReferenceCriterion)


class TestATBooleanCriterion(CriteriaTest):
    klass = ATBooleanCriterion
    title = 'Boolean Criterion'
    meta_type = 'ATBooleanCriterion'
    portal_type = 'ATBooleanCriterion'

    def test_boolean_query_true(self):
        self.dummy.Schema()['field'].set(self.dummy, 'isPrincipiaFolderish')
        self.dummy.setBool(True)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'isPrincipiaFolderish')
        self.assertEqual(query, [1, True, '1', 'True'])

    def test_boolean_query_false(self):
        self.dummy.Schema()['field'].set(self.dummy, 'isPrincipiaFolderish')
        self.dummy.setBool(False)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'isPrincipiaFolderish')
        self.assertEqual(query,
                         [0, '', False, '0', 'False', None, (), [], {}, MV])

tests.append(TestATBooleanCriterion)


class TestATPortalTypeCriterion(CriteriaTest):
    klass = ATPortalTypeCriterion
    title = 'Portal Types Criterion'
    meta_type = 'ATPortalTypeCriterion'
    portal_type = 'ATPortalTypeCriterion'

    #Same as list criterion but without operator and with special vocabulary
    def test_portaltype_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'portal_type')
        self.dummy.setValue(('Document', 'Folder', 'Topic'))
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'portal_type')
        self.assertEqual(query, ('Document', 'Folder', 'Topic'))

    def test_vocabulary(self):
        #Should return standard types, but not blacklisted types
        self.dummy.Schema()['field'].set(self.dummy, 'portal_types')
        self.assertTrue('Document' in self.dummy.getCurrentValues().keys())
        self.assertTrue('ATSimpleStringCriterion' not in self.dummy.getCurrentValues().keys())

    def test_vocabulary_sorts_by_title(self):
        #Should return standard types, but not blacklisted types
        self.dummy.Schema()['field'].set(self.dummy, 'Type')
        type_names = self.dummy.getCurrentValues().values()
        self.assertTrue(type_names.index('Page') > type_names.index('Event'))
        self.dummy.Schema()['field'].set(self.dummy, 'portal_types')
        type_names = self.dummy.getCurrentValues().values()
        self.assertTrue(type_names.index('Page') > type_names.index('Event'))

    def test_type_ids_names(self):
        self.dummy.Schema()['field'].set(self.dummy, 'Type')
        type_ids = self.dummy.getCurrentValues().keys()
        type_names = self.dummy.getCurrentValues().values()
        self.assertTrue('Page' in type_ids)
        self.assertTrue('Page' in type_names)
        self.assertFalse('Document' in type_ids)
        self.assertFalse('Document' in type_names)
        # use type title everytime
        self.dummy.Schema()['field'].set(self.dummy, 'portal_type')
        type_ids = self.dummy.getCurrentValues().keys()
        type_names = self.dummy.getCurrentValues().values()
        self.assertTrue('Document' in type_ids)
        self.assertTrue('Page' in type_names)
        self.assertFalse('Page' in type_ids)
        self.assertFalse('Document' in type_names)
        # ensure that blacklisted types aren't included
        self.assertFalse('Simple String Criterion' in type_ids)


tests.append(TestATPortalTypeCriterion)


class TestATCurrentAuthorCriterion(CriteriaTest):
    klass = ATCurrentAuthorCriterion
    title = 'Current Author Criterion'
    meta_type = 'ATCurrentAuthorCriterion'
    portal_type = 'ATCurrentAuthorCriterion'

    def afterSetUp(self):
        CriteriaTest.afterSetUp(self)
        self.portal.acl_users._doAddUser('member', 'secret', ['Member'], [])
        self.portal.acl_users._doAddUser('reviewer', 'secret', ['Reviewer'], [])

    def test_author_query(self):
        self.dummy.Schema()['field'].set(self.dummy, 'creator')
        self.login('member')
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'creator')
        self.assertEqual(query, 'member')
        self.login('reviewer')
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'creator')
        self.assertEqual(query, 'reviewer')

tests.append(TestATCurrentAuthorCriterion)


class TestATRelativePathCriterion(CriteriaTest):
    klass = ATRelativePathCriterion
    title = 'Relative Path Criterion'
    meta_type = 'ATRelativePathCriterion'
    portal_type = 'ATRelativePathCriterion'

    def afterSetUp(self):
        CriteriaTest.afterSetUp(self)
        self.setRoles(['Manager'])
        # build folder structure
        self.portal.invokeFactory('Folder', 'folderA')
        self.portal.invokeFactory('Folder', 'folderB')
        self.portal.folderA.invokeFactory('Folder', 'folderA1')
        self.portal.folderB.invokeFactory('Folder', 'folderB1')

        # create topic in folderA1
        self.portal.folderA.folderA1.invokeFactory('Topic', 'new_topic', title='New Topic')

        self.topic = self.portal.folderA.folderA1.new_topic
        # Add a path criterion
        self.path_crit = self.topic.addCriterion('path', 'ATRelativePathCriterion')

    def test_relative_path_query1(self):
        self.path_crit.setRelativePath('..')  # should give the parent==folderA1
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone/folderA/folderA1', 'depth': 1}),))

    def test_relative_path_query2(self):
        self.path_crit.setRelativePath('../..')  # should give folderA
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone/folderA', 'depth': 1}),))

    def test_relative_path_query3(self):
        self.path_crit.setRelativePath('../../..')  # should give the /plone (portal)
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone', 'depth': 1}),))

    def test_relative_path_query4(self):
        self.path_crit.setRelativePath('../../../../../../..')  # should give the /plone (portal): cannot go higher than the portal
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone', 'depth': 1}),))

    def test_relative_path_query5(self):
        self.path_crit.setRelativePath('../../../folderB')  # should give folderB
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone/folderB', 'depth': 1}),))

    def test_relative_path_query6(self):
        self.path_crit.setRelativePath('/folderB')  # should give folderB also (absolute paths are supported)
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone/folderB', 'depth': 1}),))

    def test_relative_path_query7(self):
        self.path_crit.setRelativePath('../../folderA1/../../folderB/folderB1/..')   # should give folderB
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone/folderB', 'depth': 1}),))

    def test_relative_path_query8(self):
        self.path_crit.setRelativePath('.')  # should give the new_topic
        self.assertTrue(self.path_crit.getCriteriaItems() == (('path', {'query': '/plone/folderA/folderA1/new_topic', 'depth': 1}),))

    def test_relative_path_query9(self):
        # Acquisition can mess us up, for example when a BrowserView
        # is in the acquisition chain, like in
        # plone.app.content.browser.foldercontents
        self.path_crit.setRelativePath('..')  # should give the parent==folderA1
        from Products.Five import BrowserView
        view = BrowserView(self.topic, self.topic.REQUEST)
        criterion = view.context.getCriterion('path_ATRelativePathCriterion')
        self.assertTrue(criterion.getCriteriaItems() == (('path', {'query': '/plone/folderA/folderA1', 'depth': 1}),))


tests.append(TestATRelativePathCriterion)


class TestATPathCriterion(CriteriaTest):
    klass = ATPathCriterion
    title = 'Path Criterion'
    meta_type = 'ATPathCriterion'
    portal_type = 'ATPathCriterion'

    def test_path_query(self):
        # ensure that the path and recurse settings result in a proper query
        self.dummy.Schema()['field'].set(self.dummy, 'path')
        self.folder.invokeFactory('Document', 'doc1')
        uid = self.folder.doc1.UID()
        self.dummy.setValue((uid,))
        self.dummy.setRecurse(True)
        items = self.dummy.getCriteriaItems()
        self.assertEqual(len(items), 1)
        query = items[0][1]
        field = items[0][0]
        self.assertEqual(field, 'path')
        self.assertEqual(tuple(query['query']), ('/plone/Members/test_user_1_/doc1',))
        self.assertEqual(query['depth'], -1)
        self.dummy.setRecurse(False)
        items = self.dummy.getCriteriaItems()
        query = items[0][1]
        self.assertEqual(query['depth'], 1)

    # Some reference errors were making this impossible
    def test_path_criteria_can_be_removed(self):
        self.setRoles(['Manager', 'Member'])
        self.folder.invokeFactory('Topic', 'new_topic', title='New Topic')
        topic = self.folder.new_topic
        # Add a path criterion
        path_crit = topic.addCriterion('path', 'ATPathCriterion')
        # Give it a reference
        path_crit.setValue([topic.UID()])
        # The error is masked when not in debug mode or as manager, though it
        # still has consequences
        from App.config import getConfiguration
        config = getConfiguration()
        orig_debug = config.debug_mode
        config.debug_mode = True
        self.setRoles(['Member'])
        # Delete the topic
        try:
            self.folder.manage_delObjects(['new_topic'])
        except AttributeError:
            config.debug_mode = orig_debug
            self.fail("Deleting a topic with path criteria raises an error!")
        config.debug_mode = orig_debug

tests.append(TestATPathCriterion)


class TestCriterionRegistry(atcttestcase.ATCTSiteTestCase):

    def afterSetUp(self):
        from Products.ATContentTypes.criteria import _criterionRegistry
        atcttestcase.ATCTSiteTestCase.afterSetUp(self)
        self.crit_registry = _criterionRegistry

    def testRegisterCriteria(self):
        # Ensure that the criteria registering and unregistering mechanism
        # works as expected
        # check if the expected criteria is there
        self.assertTrue(ATDateCriteria in self.crit_registry.listCriteria())
        self.assertTrue(self.crit_registry.indicesByCriterion('ATFriendlyDateCriteria'))
        # remove and ensure that it was removed
        self.crit_registry.unregister(ATDateCriteria)
        self.assertFalse(ATDateCriteria in self.crit_registry.listCriteria())
        # add and ensure that it was added
        self.crit_registry.register(ATDateCriteria, ('Bogus Index',))
        self.assertTrue(ATDateCriteria in self.crit_registry.listCriteria())
        self.assertEqual(self.crit_registry.indicesByCriterion('ATFriendlyDateCriteria'),
                                                        ('Bogus Index',))

    def testCriteriaIndexLookupOnBadIndex(self):
        # Make sure we don't throw errors when someone has a non-default index
        # type in their catalog.
        self.crit_registry.criteriaByIndex('My Bad Index Type')

tests.append(TestCriterionRegistry)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from UserDict import UserDict
from Products.Archetypes.atapi import registerType
from Products.Archetypes.ClassGen import generateClass
from Products.ATContentTypes.config import PROJECTNAME
from types import StringType

from Products.ATContentTypes.interfaces import IATTopicCriterion
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.interfaces import IATTopicSortCriterion

ALL_INDICES = ('DateIndex', 'DateRangeIndex', 'FieldIndex', 'KeywordIndex',
               'PathIndex', 'TextIndex', 'TextIndexNG2', 'TextIndexNG3',
               'TopicIndex', 'ZCTextIndex', 'NavtreeIndexNG',
               'ExtendedPathIndex', 'BooleanIndex', 'UUIDIndex')

SORT_INDICES = ('DateIndex', 'DateRangeIndex', 'FieldIndex', 'KeywordIndex',
                'PathIndex', 'GopipIndex')
# TextIndex, TopicIndex, ZCTextIndex, TextIndexNG2, NavtreeIndexNG
# are not usable to sort as they do not have 'keyForDocument' attribute

DATE_INDICES = ('DateIndex', 'DateRangeIndex', 'FieldIndex')

# Indices that can take a list of values
LIST_INDICES = ('FieldIndex', 'KeywordIndex', 'PathIndex', 'NavtreeIndexNG',
                  'ExtendedPathIndex', 'TopicIndex')

TEXT_INDICES = ('TextIndex', 'TextIndexNG2', 'TextIndexNG3', 'ZCTextIndex')

# Indices that can take a simple string
STRING_INDICES = LIST_INDICES + TEXT_INDICES

# Indices that may hold AT reference data
REFERENCE_INDICES = ('FieldIndex', 'KeywordIndex')
FIELD_INDICES = ('FieldIndex',)
PATH_INDICES = ('PathIndex', 'ExtendedPathIndex')


class _CriterionRegistry(UserDict):
    """Registry for criteria """

    def __init__(self, *args, **kwargs):
        UserDict.__init__(self, *args, **kwargs)
        self.index2criterion = {}
        self.criterion2index = {}
        self.portaltypes = {}

    def register(self, criterion, indices):
        if type(indices) is StringType:
            indices = (indices,)
        indices = tuple(indices)

        if indices == ():
            indices = ALL_INDICES

        assert IATTopicCriterion.implementedBy(criterion)
        registerType(criterion, PROJECTNAME)

        crit_id = criterion.meta_type
        self[crit_id] = criterion
        self.portaltypes[criterion.portal_type] = criterion

        self.criterion2index[crit_id] = indices
        for index in indices:
            value = self.index2criterion.get(index, ())
            self.index2criterion[index] = value + (crit_id,)

    def unregister(self, criterion):
        crit_id = criterion.meta_type
        self.pop(crit_id)
        self.criterion2index.pop(crit_id)
        for (index, value) in self.index2criterion.items():
            if id in value:
                valuelist = list(value)
                del valuelist[valuelist.index(crit_id)]
                self.index2criterion[index] = tuple(valuelist)

    def listTypes(self):
        return self.keys()

    def listSortTypes(self):
        return [key for key in self.keys()
                    if IATTopicSortCriterion.implementedBy(self[key])]

    def listSearchTypes(self):
        return [key for key in self.keys()
                    if IATTopicSearchCriterion.implementedBy(self[key])]

    def listCriteria(self):
        return self.values()

    def indicesByCriterion(self, criterion):
        return self.criterion2index[criterion]

    def criteriaByIndex(self, index):
        try:
            return self.index2criterion[index]
        except KeyError:
            return ()

    def getPortalTypes(self):
        return tuple(self.portaltypes.keys())

_criterionRegistry = _CriterionRegistry()
registerCriterion = _criterionRegistry.register
unregisterCriterion = _criterionRegistry.unregister

__all__ = ('registerCriterion', 'ALL_INDICES', 'DATE_INDICES', 'STRING_INDICES',
           'LIST_INDICES', 'SORT_INDICES', )

# criteria
from Products.ATContentTypes.criteria.boolean import ATBooleanCriterion
from Products.ATContentTypes.criteria.date import ATDateCriteria
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterion
from Products.ATContentTypes.criteria.list import ATListCriterion
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterion
from Products.ATContentTypes.criteria.reference import ATReferenceCriterion
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterion
from Products.ATContentTypes.criteria.simplestring import ATSimpleStringCriterion
from Products.ATContentTypes.criteria.sort import ATSortCriterion
from Products.ATContentTypes.criteria.currentauthor import ATCurrentAuthorCriterion
from Products.ATContentTypes.criteria.path import ATPathCriterion
from Products.ATContentTypes.criteria.relativepath import ATRelativePathCriterion



from cStringIO import StringIO
from zope.interface import implements

from DateTime import DateTime
from App.class_init import InitializeClass

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.ATContentTypes.interfaces import ICalendarSupport

PRODID = "-//AT Content Types//AT Event//EN"

# iCal header and footer
ICS_HEADER = """\
BEGIN:VCALENDAR
PRODID:%(prodid)s
VERSION:2.0
METHOD:PUBLISH
"""

ICS_FOOTER = """\
END:VCALENDAR
"""

# Note: a previous version of event start set "SEQUENCE:0"
# That's not necessary unless we're supporting recurrence.

# iCal event
ICS_EVENT_START = """\
BEGIN:VEVENT
DTSTAMP:%(dtstamp)s
CREATED:%(created)s
UID:ATEvent-%(uid)s
LAST-MODIFIED:%(modified)s
SUMMARY:%(summary)s
DTSTART:%(startdate)s
DTEND:%(enddate)s
"""

# Note: a previous version of event end set "TRANSP:OPAQUE", which would cause
# blocking of the time slot. That's not appropriate for an event
# calendar.
# Also, previous version set "PRIORITY:3", which the RFC interprets as a high
# priority. In absence of a priority field in the event, there's no justification
# for that.

ICS_EVENT_END = """\
CLASS:PUBLIC
END:VEVENT
"""

# vCal header and footer
VCS_HEADER = """\
BEGIN:VCALENDAR
PRODID:%(prodid)s
VERSION:1.0
"""

VCS_FOOTER = """\
END:VCALENDAR
"""

# vCal event
VCS_EVENT_START = """\
BEGIN:VEVENT
DTSTART:%(startdate)s
DTEND:%(enddate)s
DCREATED:%(created)s
UID:ATEvent-%(uid)s
SEQUENCE:0
LAST-MODIFIED:%(modified)s
SUMMARY:%(summary)s
"""

VCS_EVENT_END = """\
PRIORITY:3
TRANSP:0
END:VEVENT
"""


class CalendarSupportMixin:
    """Mixin class for iCal/vCal support
    """

    implements(ICalendarSupport)

    security = ClassSecurityInfo()

    actions = ({
        'id': 'ics',
        'name': 'iCalendar',
        'action': 'string:${object_url}/ics_view',
        'permissions': (View, ),
        'category': 'document_actions',
         },
         {
        'id': 'vcs',
        'name': 'vCalendar',
        'action': 'string:${object_url}/vcs_view',
        'permissions': (View, ),
        'category': 'document_actions',
         },
    )

    _at_action_icons = ({
        'category': 'plone',
        'action_id': 'ics',
        'icon_expr': 'ical_icon.gif',
        'title': 'iCalendar export',
        'priority': 0,
        },
        {
        'category': 'plone',
        'action_id': 'vcs',
        'icon_expr': 'vcal_icon.gif',
        'title': 'vCalendar export',
        'priority': 0,
        },
        )

    security.declareProtected(View, 'getICal')
    def getICal(self):
        """get iCal data
        """
        out = StringIO()
        map = {
            'dtstamp': rfc2445dt(DateTime()),
            'created': rfc2445dt(DateTime(self.CreationDate())),
            'uid': self.UID(),
            'modified': rfc2445dt(DateTime(self.ModificationDate())),
            'summary': vformat(self.Title()),
            'startdate': rfc2445dt(self.start()),
            'enddate': rfc2445dt(self.end()),
            }
        out.write(ICS_EVENT_START % map)

        description = self.Description()
        if description:
            out.write(foldLine('DESCRIPTION:%s\n' % vformat(description)))

        location = self.getLocation()
        if location:
            out.write('LOCATION:%s\n' % vformat(location))

        subject = self.Subject()
        if subject:
            out.write('CATEGORIES:%s\n' % ', '.join(subject))

        # TODO  -- NO! see the RFC; ORGANIZER field is not to be used for non-group-scheduled entities
        #ORGANIZER;CN=%(name):MAILTO=%(email)
        #ATTENDEE;CN=%(name);ROLE=REQ-PARTICIPANT:mailto:%(email)

        cn = []
        contact = self.contact_name()
        if contact:
            cn.append(contact)
        phone = self.contact_phone()
        if phone:
            cn.append(phone)
        email = self.contact_email()
        if email:
            cn.append(email)
        if cn:
            out.write('CONTACT:%s\n' % vformat(', '.join(cn)))

        url = self.event_url()
        if url:
            out.write('URL:%s\n' % url)

        out.write(ICS_EVENT_END)
        return out.getvalue()

    security.declareProtected(View, 'ics_view')
    def ics_view(self, REQUEST, RESPONSE):
        """iCalendar output
        """
        RESPONSE.setHeader('Content-Type', 'text/calendar')
        RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s.ics"' % self.getId())
        out = StringIO()
        out.write(ICS_HEADER % {'prodid': PRODID})
        out.write(self.getICal())
        out.write(ICS_FOOTER)
        return n2rn(out.getvalue())

    security.declareProtected(View, 'getVCal')
    def getVCal(self):
        """get vCal data
        """
        out = StringIO()
        map = {
            'dtstamp': rfc2445dt(DateTime()),
            'created': rfc2445dt(DateTime(self.CreationDate())),
            'uid': self.UID(),
            'modified': rfc2445dt(DateTime(self.ModificationDate())),
            'summary': vformat(self.Title()),
            'startdate': rfc2445dt(self.start()),
            'enddate': rfc2445dt(self.end()),
            }
        out.write(VCS_EVENT_START % map)
        description = self.Description()
        if description:
            out.write(foldLine('DESCRIPTION:%s\n' % vformat(description)))
        location = self.getLocation()
        if location:
            out.write('LOCATION:%s\n' % vformat(location))
        out.write(VCS_EVENT_END)
        # TODO
        # Insert missing code here :]
        return out.getvalue()

    security.declareProtected(View, 'vcs_view')
    def vcs_view(self, REQUEST, RESPONSE):
        """vCalendar output
        """
        RESPONSE.setHeader('Content-Type', 'text/x-vCalendar')
        RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s.vcs"' % self.getId())
        out = StringIO()
        out.write(VCS_HEADER % {'prodid': PRODID})
        out.write(self.getVCal())
        out.write(VCS_FOOTER)
        return n2rn(out.getvalue())

InitializeClass(CalendarSupportMixin)


def vformat(s):
    # return string with escaped commas and semicolons
    # NOTE: RFC 2445 specifies "a COLON character in a 'TEXT' property value
    # SHALL NOT be escaped with a BACKSLASH character." So watch out for
    # non-TEXT values, should they be introduced later in this code!
    return s.strip().replace(', ', '\, ').replace(';', '\;')


def n2rn(s):
    return s.replace('\n', '\r\n')


def rfc2445dt(dt):
    # return UTC in RFC2445 format YYYYMMDDTHHMMSSZ
    return dt.HTML4().replace('-', '').replace(':', '')


def foldLine(s):
    # returns string folded per RFC2445 (each line must be less than 75 octets)
    # This code is a minor modification of MakeICS.py, available at:
    # http://www.zope.org/Members/Feneric/MakeICS/

    lineLen = 70

    workStr = s.strip().replace('\r\n', '\n').replace('\r', '\n').replace('\n', '\\n')
    numLinesToBeProcessed = len(workStr) / lineLen
    startingChar = 0
    res = ''
    while numLinesToBeProcessed >= 1:
        res = '%s%s\n ' % (res, workStr[startingChar:startingChar + lineLen])
        startingChar += lineLen
        numLinesToBeProcessed -= 1
    return '%s%s\n' % (res, workStr[startingChar:])



# -*- coding: UTF-8 -*-

from Testing import ZopeTestCase

from Products.ATContentTypes.tests.atcttestcase import ATCTFunctionalSiteTestCase

FILES = ['events.txt', 'editing.txt']


def test_suite():
    import unittest
    suite = unittest.TestSuite()
    from Testing.ZopeTestCase import FunctionalDocFileSuite as FileSuite
    for file in FILES:
        suite.addTest(FileSuite(file, package="Products.ATContentTypes.tests",
                                test_class=ATCTFunctionalSiteTestCase)
                     )
    return suite



from types import TupleType

from zope.interface import implements

from ZPublisher.HTTPRequest import HTTPRequest
from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from ComputedAttribute import ComputedAttribute

from Products.CMFDefault.utils import SimpleHTMLParser
from Products.GenericSetup.interfaces import IDAVAware

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import TextField
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import RichWidget
from Products.Archetypes.atapi import BooleanWidget
from Products.Archetypes.atapi import RFC822Marshaller
from Products.Archetypes.atapi import AnnotationStorage

from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.base import translateMimetypeAlias
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
from Products.ATContentTypes.interfaces import IATDocument

from Products.ATContentTypes import ATCTMessageFactory as _

ATDocumentSchema = ATContentTypeSchema.copy() + Schema((
    TextField('text',
              required=False,
              searchable=True,
              primary=True,
              storage=AnnotationStorage(migrate=True),
              validators=('isTidyHtmlWithCleanup',),
              #validators=('isTidyHtml',),
              default_output_type='text/x-html-safe',
              widget=RichWidget(
                        description='',
                        label=_(u'label_body_text', default=u'Body Text'),
                        rows=25,
                        allow_file_upload=zconf.ATDocument.allow_document_upload),
    ),

    BooleanField('presentation',
        required=False,
        languageIndependent=True,
        widget=BooleanWidget(
            label=_(
                u'help_enable_presentation',
                default=u'Presentation mode'),
            description=_(
                u'help_enable_presentation_description',
                default=u'If selected, this will give users the ability to view the contents as presentation slides.')
            ),
    ),

    BooleanField('tableContents',
        required=False,
        languageIndependent=True,
        widget=BooleanWidget(
            label=_(
                u'help_enable_table_of_contents',
                default=u'Table of contents'),
            description=_(
                u'help_enable_table_of_contents_description',
                default=u'If selected, this will show a table of contents at the top of the page.')
            ),
    )),
    marshall=RFC822Marshaller()
    )

ATDocumentSchema['description'].widget.label = \
    _(u'label_summary', default=u'Summary')

finalizeATCTSchema(ATDocumentSchema)
# moved schema setting after finalizeATCTSchema, so the order of the fieldsets
# is preserved
ATDocumentSchema.changeSchemataForField('presentation', 'settings')
ATDocumentSchema.changeSchemataForField('tableContents', 'settings')


class ATDocumentBase(ATCTContent, HistoryAwareMixin):
    """A page in the site. Can contain rich text."""

    security = ClassSecurityInfo()
    cmf_edit_kws = ('text_format',)

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, stx_level='ignored'):
        """CMF compatibility method
        """
        return self.getText()

    security.declareProtected(ModifyPortalContent, 'EditableBody')
    def EditableBody(self):
        """CMF compatibility method
        """
        return self.getRawText()

    security.declareProtected(ModifyPortalContent, 'setFormat')
    def setFormat(self, value):
        """CMF compatibility method

        The default mutator is overwritten to:

          o add a conversion from stupid CMF content type (e.g. structured-text)
            to real mime types used by MTR.

          o Set format to default format if value is empty

        """
        if not value:
            value = zconf.ATDocument.default_content_type
        else:
            value = translateMimetypeAlias(value)
        ATCTContent.setFormat(self, value)

    security.declareProtected(ModifyPortalContent, 'setText')
    def setText(self, value, **kwargs):
        """Body text mutator

        * hook into mxTidy an replace the value with the tidied value
        """
        field = self.getField('text')

        # When an object is initialized the first time we have to
        # set the filename and mimetype.
        if not value and not field.getRaw(self):
            if 'mimetype' in kwargs and kwargs['mimetype']:
                field.setContentType(self, kwargs['mimetype'])
            if 'filename' in kwargs and kwargs['filename']:
                field.setFilename(self, kwargs['filename'])

        # hook for mxTidy / isTidyHtmlWithCleanup validator
        tidyOutput = self.getTidyOutput(field)
        if tidyOutput:
            value = tidyOutput

        field.set(self, value, **kwargs)  # set is ok

    text_format = ComputedAttribute(ATCTContent.getContentType, 1)

    security.declarePrivate('guessMimetypeOfText')
    def guessMimetypeOfText(self):
        """For ftp/webdav upload: get the mimetype from the id and data
        """
        mtr = getToolByName(self, 'mimetypes_registry')
        id = self.getId()
        data = self.getRawText()
        ext = id.split('.')[-1]

        if ext != id:
            mimetype = mtr.classify(data, filename=ext)
        else:
            # no extension
            mimetype = mtr.classify(data)

        if not mimetype or (type(mimetype) is TupleType and not len(mimetype)):
            # nothing found
            return None

        if type(mimetype) is TupleType and len(mimetype):
            mimetype = mimetype[0]
        return mimetype.normalized()

    security.declarePrivate('getTidyOutput')
    def getTidyOutput(self, field):
        """Get the tidied output for a specific field from the request
        if available
        """
        request = getattr(self, 'REQUEST', None)
        if request is not None and isinstance(request, HTTPRequest):
            tidyAttribute = '%s_tidier_data' % field.getName()
            return request.get(tidyAttribute, None)

    def _notifyOfCopyTo(self, container, op=0):
        """Override this to store a flag when we are copied, to be able
        to discriminate the right thing to do in manage_afterAdd here
        below.
        """
        self._v_renamed = 1
        return ATCTContent._notifyOfCopyTo(self, container, op=op)

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Fix text when created througt webdav
        Guess the right mimetype from the id/data
        """
        ATCTContent.manage_afterAdd(self, item, container)
        field = self.getField('text')

        # hook for mxTidy / isTidyHtmlWithCleanup validator
        tidyOutput = self.getTidyOutput(field)
        if tidyOutput:
            if hasattr(self, '_v_renamed'):
                mimetype = field.getContentType(self)
                del self._v_renamed
            else:
                mimetype = self.guessMimetypeOfText()
            if mimetype:
                field.set(self, tidyOutput, mimetype=mimetype)  # set is ok
            elif tidyOutput:
                field.set(self, tidyOutput)  # set is ok

    security.declarePrivate('cmf_edit')
    def cmf_edit(self, text_format, text, file='', safety_belt='', **kwargs):
        assert file == '', 'file currently not supported'  # XXX
        self.setText(text, mimetype=translateMimetypeAlias(text_format))
        self.update(**kwargs)

    security.declarePrivate('manage_afterPUT')
    def manage_afterPUT(self, data, marshall_data, file, context, mimetype,
                        filename, REQUEST, RESPONSE):
        """After webdav/ftp PUT method

        Set title according to the id on webdav/ftp PUTs.
        """

        if '' == data:
            file.seek(0)
            content = file.read(65536)
        else:
            content = data

        if -1 != content.lower().find("<html"):
            parser = SimpleHTMLParser()
            parser.feed(content)
            if parser.title:
                self.setTitle(parser.title)
            return

        ATCTContent.manage_afterPUT(self, data, marshall_data, file,
                                    context, mimetype, filename, REQUEST,
                                    RESPONSE)


class ATDocument(ATDocumentBase):
    """A page in the site. Can contain rich text."""

    schema = ATDocumentSchema

    portal_type = 'Document'
    archetype_name = 'Page'
    _atct_newTypeFor = {'portal_type': 'CMF Document', 'meta_type': 'Document'}
    assocMimetypes = ('application/xhtml+xml', 'message/rfc822', 'text/*',)
    assocFileExt = ('txt', 'stx', 'rst', 'rest', 'py',)

    implements(IATDocument, IDAVAware)

registerATCT(ATDocument, PROJECTNAME)



import logging
from cStringIO import StringIO

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from AccessControl import ClassSecurityInfo
from ExtensionClass import Base
from DateTime import DateTime
from App.class_init import InitializeClass
from OFS.Image import Image as OFSImage
from OFS.Image import Pdata

from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.config import HAS_PIL
from Products.ATContentTypes import ATCTMessageFactory as _

# third party extension
import exif

# the following code is based on the rotation code of Photo
if HAS_PIL:
    import PIL.Image

LOG = logging.getLogger('ATCT.image')

# transpose constants, taken from PIL.Image to maintain compatibilty
FLIP_LEFT_RIGHT = 0
FLIP_TOP_BOTTOM = 1
ROTATE_90 = 2
ROTATE_180 = 3
ROTATE_270 = 4


TRANSPOSE_MAP = {
    FLIP_LEFT_RIGHT: _(u'Flip around vertical axis'),
    FLIP_TOP_BOTTOM: _(u'Flip around horizontal axis'),
    ROTATE_270: _(u'Rotate 90 clockwise'),
    ROTATE_180: _(u'Rotate 180'),
    ROTATE_90: _(u'Rotate 90 counterclockwise'),
   }

AUTO_ROTATE_MAP = {
    0: None,
    90: ROTATE_270,
    180: ROTATE_180,
    270: ROTATE_90,
    }


class ATCTImageTransform(Base):
    """Base class for images containing transformation and exif functions

    * exif information
    * image rotation
    """

    security = ClassSecurityInfo()

    security.declarePrivate('getImageAsFile')
    def getImageAsFile(self, img=None, scale=None):
        """Get the img as file like object
        """
        if img is None:
            f = self.getField('image')
            img = f.getScale(self, scale)
        # img.data contains the image as string or Pdata chain
        data = None
        if isinstance(img, OFSImage):
            data = str(img.data)
        elif isinstance(img, Pdata):
            data = str(img)
        elif isinstance(img, str):
            data = img
        elif isinstance(img, file) or (hasattr(img, 'read') and
          hasattr(img, 'seek')):
            img.seek(0)
            return img
        if data:
            return StringIO(data)
        else:
            return None

    # image related code like exif and rotation
    # partly based on CMFPhoto

    security.declareProtected(View, 'getEXIF')
    def getEXIF(self, img=None, refresh=False):
        """Get the exif informations of the file

        The information is cached in _v_image_exif
        """
        cache = '_image_exif'

        if refresh:
            setattr(self, cache, None)

        exif_data = getattr(self, cache, None)

        if exif_data is None or not isinstance(exif_data, dict):
            io = self.getImageAsFile(img, scale=None)
            if io is not None:
                # some cameras are naughty :(
                try:
                    io.seek(0)
                    exif_data = exif.process_file(io, debug=False)
                except:
                    LOG.error('Failed to process EXIF information', exc_info=True)
                    exif_data = {}
                # seek to 0 and do NOT close because we might work
                # on a file upload which is required later
                io.seek(0)
                # remove some unwanted elements lik thumb nails
                for key in ('JPEGThumbnail', 'TIFFThumbnail',
                            'MakerNote JPEGThumbnail'):
                    if key in exif_data:
                        del exif_data[key]

        if not exif_data:
            # alawys return a dict
            exif_data = {}
        # set the EXIF cache even if the image has returned an empty
        # dict. This prevents regenerating the exif every time if an
        # image doesn't have exif information.
        setattr(self, cache, exif_data)
        return exif_data

    security.declareProtected(View, 'getEXIFOrientation')
    def getEXIFOrientation(self):
        """Get the rotation and mirror orientation from the EXIF data

        Some cameras are storing the informations about rotation and mirror in
        the exif data. It can be used for autorotation.
        """
        exif = self.getEXIF()
        mirror = 0
        rotation = 0
        code = exif.get('Image Orientation', None)

        if code is None:
            return (mirror, rotation)

        try:
            code = int(code)
        except ValueError:
            return (mirror, rotation)

        if code in (2, 4, 5, 7):
            mirror = 1
        if code in (1, 2):
            rotation = 0
        elif code in (3, 4):
            rotation = 180
        elif code in (5, 6):
            rotation = 90
        elif code in (7, 8):
            rotation = 270

        return (mirror, rotation)

    security.declareProtected(View, 'getEXIFOrigDate')
    def getEXIFOrigDate(self):
        """Get the EXIF DateTimeOriginal from the image (or None)
        """
        exif_data = self.getEXIF()
        raw_date = exif_data.get('EXIF DateTimeOriginal', None)
        if raw_date is not None:
            # some cameras are naughty ...
            try:
                return DateTime(str(raw_date))
            except:
                LOG.error('Failed to parse exif date %s' % raw_date, exc_info=True)
        return None

    security.declareProtected(ModifyPortalContent, 'transformImage')
    def transformImage(self, method, REQUEST=None):
        """
        Transform an Image:
            FLIP_LEFT_RIGHT
            FLIP_TOP_BOTTOM
            ROTATE_90 (rotate counterclockwise)
            ROTATE_180
            ROTATE_270 (rotate clockwise)
        """
        try:
            method = int(method)
        except ValueError:
            method = int(REQUEST.form.get('method'))
        if method not in TRANSPOSE_MAP:
            raise RuntimeError, "Unknown method %s" % method

        target = self.absolute_url() + '/atct_image_transform'

        if not HAS_PIL:
            if REQUEST:
                REQUEST.RESPONSE.redirect(target)

        image = self.getImageAsFile()
        image2 = StringIO()

        if image is not None:
            img = PIL.Image.open(image)
            del image
            fmt = img.format
            img = img.transpose(method)
            img.save(image2, fmt, quality=zconf.pil_config.quality)

            field = self.getField('image')
            mimetype = field.getContentType(self)
            filename = field.getFilename(self)

            # because AT tries to get mimetype and filename from a file like
            # object by attribute access I'm passing a string along
            self.setImage(image2.getvalue(), mimetype=mimetype,
                          filename=filename, refresh_exif=False)
            self.reindexObject()

        if REQUEST:
            REQUEST.RESPONSE.redirect(target)

    security.declareProtected(ModifyPortalContent, 'autoTransformImage')
    def autoTransformImage(self, REQUEST=None):
        """Auto transform image according to EXIF data

        Note: isn't using mirror
        """
        target = self.absolute_url() + '/atct_image_transform'
        mirror, rotation = self.getEXIFOrientation()
        transform = None
        if rotation:
            transform = AUTO_ROTATE_MAP.get(rotation, None)
            if transform is not None:
                self.transformImage(transform)
        if REQUEST:
            REQUEST.RESPONSE.redirect(target)
        else:
            return mirror, rotation, transform

    security.declareProtected(View, 'getTransformMap')
    def getTransformMap(self):
        """Get map for tranforming the image
        """
        return [{'name': n, 'value': v} for v, n in TRANSPOSE_MAP.items()]

    security.declareProtected(View, 'hasPIL')
    def hasPIL(self):
        """Is PIL installed?
        """
        return HAS_PIL

InitializeClass(ATCTImageTransform)



from zope.interface import implements

from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo

from Products.ATContentTypes.criteria import registerCriterion, \
                                             LIST_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion

ATCurrentAuthorSchema = ATBaseCriterionSchema


class ATCurrentAuthorCriterion(ATBaseCriterion):
    """A criterion that searches for the currently logged in user's id"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATCurrentAuthorSchema
    meta_type = 'ATCurrentAuthorCriterion'
    archetype_name = 'Current Author Criterion'
    shortDesc = 'Restrict to current user'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = []

        user = getToolByName(self, 'portal_membership').getAuthenticatedMember().getId()

        if user is not '':
            result.append((self.Field(), user))

        return tuple(result)

registerCriterion(ATCurrentAuthorCriterion, LIST_INDICES)



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase
from DateTime import DateTime
import Missing

tests = []


class TestFormatCatalogMetadata(atcttestcase.ATCTSiteTestCase):

    def afterSetUp(self):
        atcttestcase.ATCTSiteTestCase.afterSetUp(self)
        self.script = self.portal.formatCatalogMetadata

    def testFormatDate(self):
        date = '2005-11-02 13:52:25'
        toLocalizedTime = self.portal.toLocalizedTime
        self.assertEqual(self.script(date),
                         toLocalizedTime(date, long_format=True))
        self.assertEqual(self.script(DateTime(date)),
                         toLocalizedTime(date, long_format=True))

    def testFormatDict(self):
        self.assertEqual(self.script({'a': 1, 'b': 2}), 'a: 1, b: 2')

    def testFormatList(self):
        self.assertEqual(self.script(('a', 'b', 1, 2, 3, 4)),
                         'a, b, 1, 2, 3, 4')
        self.assertEqual(self.script(['a', 'b', 1, 2, 3, 4]),
                         'a, b, 1, 2, 3, 4')
        # this also needs to be able to handle unicode that won't encode to ascii
        ustr = 'i\xc3\xadacute'.decode('utf8')
        self.assertEqual(self.script(['a', 'b', ustr]),
                         'a, b, i\xc3\xadacute'.decode('utf8'))

    def testFormatString(self):
        self.assertEqual(self.script('fkj dsh ekjhsdf kjer'), 'fkj dsh ekjhsdf kjer')

    def testFormatTruncates(self):
        self.portal.portal_properties.site_properties.manage_changeProperties(
                            search_results_description_length=12, ellipsis='???')
        self.assertEqual(self.script('fkj dsh ekjhsdf kjer'), 'fkj dsh ekjh???')

    def testFormatStrange(self):
        self.assertEqual(self.script(None), '')
        self.assertEqual(self.script(Missing.Value()), '')

    def testUnicodeValue(self):
        """ Make sure non-ascii encodable unicode is acceptable """

        ustr = 'i\xc3\xadacute'.decode('utf8')
        self.assertEqual(self.script(ustr), ustr)

tests.append(TestFormatCatalogMetadata)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from Products.ATContentTypes.interfaces.interfaces import IATContentType
from Products.ATContentTypes.interfaces.interfaces import IHistoryAware
from Products.ATContentTypes.interfaces.interfaces import ICalendarSupport
from Products.ATContentTypes.interfaces.interfaces import ITextContent
from Products.ATContentTypes.interfaces.interfaces import ISelectableConstrainTypes
from Products.ATContentTypes.interfaces.interfaces import IATCTTool

from Products.ATContentTypes.interfaces.document import IATDocument

from Products.ATContentTypes.interfaces.event import IATEvent

from Products.ATContentTypes.interfaces.file import IFileContent
from Products.ATContentTypes.interfaces.file import IATFile

from Products.ATContentTypes.interfaces.folder import IFilterFolder
from Products.ATContentTypes.interfaces.folder import IATFolder
from Products.ATContentTypes.interfaces.folder import IATBTreeFolder

from Products.ATContentTypes.interfaces.image import IImageContent
from Products.ATContentTypes.interfaces.image import IATImage

from Products.ATContentTypes.interfaces.link import IATLink

from Products.ATContentTypes.interfaces.news import IATNewsItem

from Products.ATContentTypes.interfaces.topic import IATTopic
from Products.ATContentTypes.interfaces.topic import IATTopicCriterion
from Products.ATContentTypes.interfaces.topic import IATTopicSearchCriterion
from Products.ATContentTypes.interfaces.topic import IATTopicSortCriterion
from Products.ATContentTypes.interfaces.topic import IATCTTopicsTool



"""Migration functions for ATContentTypes 1.2. These are called during the
   usual CMFPlone migration.
"""
import logging
import transaction

from Acquisition import aq_base
from zope.component import getSiteManager

from Products.ATContentTypes.config import TOOLNAME
from Products.ATContentTypes.tool.atct import ATCTTool
from Products.ATContentTypes.interfaces import IATCTTool
from Products.CMFCore.utils import getToolByName

logger = logging.getLogger('plone.app.upgrade')


def upgradeATCTTool(portal):
    tool = getToolByName(portal, TOOLNAME, None)
    sm = getSiteManager(context=portal)
    if not hasattr(aq_base(tool), '_version'):
        # the tool already has been upgraded
        return
    # First we get all relevant old configuration and make sure we get
    # real copies of the various objects
    old_conf = {}
    old_conf['album_batch_size'] = int(getattr(tool, 'album_batch_size', 30))
    old_conf['album_image_scale'] = str(getattr(tool, 'album_image_scale', 'thumb'))
    old_conf['image_types'] = list(getattr(tool, 'image_types', ['Image', 'News Item']))
    old_conf['folder_types'] = list(getattr(tool, 'folder_types', ['Image']))
    old_conf['single_image_scale'] = str(getattr(tool, 'single_image_scale', 'preview'))
    old_conf['topic_indexes'] = tool.topic_indexes.copy()
    old_conf['topic_metadata'] = tool.topic_metadata.copy()
    old_conf['allowed_portal_types'] = tuple(tool.allowed_portal_types)

    # Remove the old tool completely
    del(tool)
    portal._delObject(TOOLNAME)
    sm.unregisterUtility(provided=IATCTTool)
    transaction.savepoint(optimistic=True)

    # Create new tool
    portal._setObject(TOOLNAME, ATCTTool())
    tool = portal.get(TOOLNAME)
    sm.registerUtility(aq_base(tool), IATCTTool)
    # And apply the configuration again
    tool._setPropValue('album_batch_size', old_conf['album_batch_size'])
    tool._setPropValue('album_image_scale', old_conf['album_image_scale'])
    tool._setPropValue('image_types', tuple(old_conf['image_types']))
    tool._setPropValue('folder_types', tuple(old_conf['folder_types']))
    tool._setPropValue('single_image_scale', old_conf['single_image_scale'])
    tool._setPropValue('allowed_portal_types', old_conf['allowed_portal_types'])

    # XXX Index and metadata should be updated instead of being reapplied
    tool._setPropValue('topic_indexes', old_conf['topic_indexes'])
    tool._setPropValue('topic_metadata', old_conf['topic_metadata'])

    transaction.savepoint(optimistic=True)

    logger.info('Upgraded the ATContentTypes tool.')



from zope.interface import implements

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import IntegerField
from Products.Archetypes.atapi import IntegerWidget
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import DisplayList

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import LIST_INDICES
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema

from Products.ATContentTypes import ATCTMessageFactory as _

DirectionOperations = DisplayList((
    ('', _(u'Equal to')),
    ('min', _(u'Greater than')),
    ('max', _(u'Less than')),
    ('min:max', _(u'Between')),
    ))

ATSimpleIntCriterionSchema = ATBaseCriterionSchema + Schema((
    IntegerField('value',
                required=1,
                mode="rw",
                write_permission=ChangeTopics,
                accessor="Value",
                mutator="setValue",
                default=None,
                widget=IntegerWidget(
                    label=_(u'label_int_criteria_value', default=u'Value'),
                    description=_(u'help_int_criteria_value',
                                  default=u'An integer number.')
                    ),
                ),
    IntegerField('value2',
                required=0,
                mode="rw",
                write_permission=ChangeTopics,
                accessor="Value2",
                mutator="setValue2",
                default=None,
                widget=IntegerWidget(
                    label=_(u'label_int_criteria_value2', default=u'Second Value'),
                    description=_(u'help_int_criteria_value2',
                                  default=u'An integer number used as the maximum value if the between direction is selected.')
                    ),
                ),
    StringField('direction',
                required=0,
                mode="rw",
                write_permission=ChangeTopics,
                default='',
                vocabulary=DirectionOperations,
                enforceVocabulary=1,
                widget=SelectionWidget(
                    label=_(u'label_int_criteria_direction', default=u'Direction'),
                    description=_(u'help_int_criteria_direction',
                                  default=u'Specify whether you want to find values lesser than, greater than, equal to, or between the chosen value(s).')
                    ),
                ),
    ))


class ATSimpleIntCriterion(ATBaseCriterion):
    """A simple int criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATSimpleIntCriterionSchema
    meta_type = 'ATSimpleIntCriterion'
    archetype_name = 'Simple Int Criterion'
    shortDesc = 'Integer value or range'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = []
        val = self.Value()
        direction = self.getDirection()
        if val or val == 0:
            if direction == 'min:max':
                val = tuple([int(val), int(self.Value2())])
            else:
                val = int(val)
            if direction:
                result.append((self.Field(), {'query': val, 'range': direction}))
            else:
                result.append((self.Field(), {'query': val}))

        return tuple(result)

    security.declareProtected(View, 'post_validate')
    def post_validate(self, REQUEST, errors):
        """Check that Value2 is set if range is set to min:max"""
        direction = REQUEST.get('direction', self.getDirection())
        val2 = REQUEST.get('value2', self.Value2())
        if direction == 'min:max' and not val2 and not val2 == 0:
            errors['value2'] = 'You must enter a second value to do a "Between" search.'
        errors['value2'] = 'You must enter a second value to do a "Between" search.'
        return errors

registerCriterion(ATSimpleIntCriterion, LIST_INDICES)



from zope.component import queryUtility
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import DisplayList
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import FIELD_INDICES
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion

from Products.ATContentTypes import ATCTMessageFactory as _

ATPortalTypeCriterionSchema = ATSelectionCriterion.schema.copy()
# and/or operator makes no sense for type selection, as no item can ever be
# two types at the same time fulfilling an AND condition
ATPortalTypeCriterionSchema.delField("operator")

val_widget = ATPortalTypeCriterionSchema['value'].widget
val_widget.description = _(u'help_portal_type_criteria_value',
                           default=u'One of the available content types.')
ATPortalTypeCriterionSchema['value'].widget = val_widget

VOCAB_ID = u'plone.app.vocabularies.ReallyUserFriendlyTypes'


class ATPortalTypeCriterion(ATSelectionCriterion):
    """A portal_types criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATPortalTypeCriterionSchema
    meta_type = 'ATPortalTypeCriterion'
    archetype_name = 'Portal Types Criterion'
    shortDesc = 'Select content types'

    security.declareProtected(View, 'getCurrentValues')
    def getCurrentValues(self):
        """Return enabled portal types"""
        vocab = queryUtility(IVocabularyFactory, name=VOCAB_ID)(self)
        portal_types = getToolByName(self, 'portal_types', None)
        result = []
        # the vocabulary returns the values sorted by their translated title
        for term in vocab._terms:
            value = term.value  # portal_type
            title = term.title  # already translated title
            if self.Field() == 'Type':
                # Switch the value from portal_type to the Title msgid
                # since that is stored in the Type-index in portal_catalog
                # TODO: we should really use the portal_type index here and
                # remove the Type index
                value = unicode(portal_types[value].Title())
            result.append((value, title))

        return DisplayList(result)

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = []
        if self.Value() is not '':
            result.append((self.Field(), self.Value()))

        return tuple(result)

registerCriterion(ATPortalTypeCriterion, FIELD_INDICES)



from Products.ATContentTypes.interfaces.interfaces import IATContentType
from zope.interface import Interface
try:
    from Products.CMFPlone.interfaces.syndication import ISyndicatable
except ImportError:
    from zope.interface import Interface as ISyndicatable


class IATTopic(IATContentType, ISyndicatable):
    """AT Topic marker interface
    """

    def listCriteriaTypes():
        """List available criteria types as dict
        """

    def listCriteriaMetaTypes():
        """List available criteria
        """

    def listSearchCriteriaTypes():
        """List available search criteria types as dict
        """

    def listSearchCriteriaMetaTypes():
        """List available search criteria
        """

    def listSortCriteriaTypes():
        """List available sort criteria types as dict
        """

    def listSortCriteriaMetaTypes():
        """List available sort criteria
        """

    def listCriteria():
        """Return a list of our criteria objects.
        """

    def listSearchCriteria():
        """Return a list of our search criteria objects.
        """

    def hasSortCriterion():
        """Tells if a sort criterai is already setup.
        """

    def getSortCriterion():
        """Return the Sort criterion if setup.
        """

    def removeSortCriterion():
        """remove the Sort criterion.
        """

    def setSortCriterion(field, reversed):
        """Set the Sort criterion.
        """

    def listAvailableFields():
        """Return a list of available fields for new criteria.
        """

    def listSortFields():
        """Return a list of available fields for sorting."""

    def listSubtopics():
        """Return a list of our subtopics.
        """

    def buildQuery():
        """Construct a catalog query using our criterion objects.
        """

    def queryCatalog(REQUEST=None, **kw):
        """Invoke the catalog using our criteria to augment any passed
            in query before calling the catalog.
        """

    def addCriterion(field, criterion_type):
        """Add a new search criterion.
        """

    def deleteCriterion(criterion_id):
        """Delete selected criterion.
        """

    def getCriterion(criterion_id):
        """Get the criterion object.
        """

    def addSubtopic(id):
        """Add a new subtopic.
        """


class IATTopicCriterion(Interface):
    """AT Topic Criterion interface
    """

    def widget(field_name, mode="view", field=None, **kwargs):
        """redefine widget() to allow seperate field_names from field
        """

    def getId():
        """get the objects id
        """

    def Type():
        """
        """

    def Description():
        """
        """

    def getCriteriaItems():
        """Return a sequence of items to be used to build the catalog query.
        """


class IATTopicSearchCriterion(IATTopicCriterion):
    """Interface for criteria used for searching
    """


class IATTopicSortCriterion(IATTopicCriterion):
    """Interface for criteria used for sorting
    """


class IATCTTopicsTool(Interface):
    """Mixin class for providing features to customize the display of topics
    """



from zope.interface import implements

from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import ImageField
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import TextField
from Products.Archetypes.atapi import ImageWidget
from Products.Archetypes.atapi import RichWidget
from Products.Archetypes.atapi import StringWidget
from Products.Archetypes.atapi import RFC822Marshaller
from Products.Archetypes.atapi import AnnotationStorage

from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import translateMimetypeAlias
from Products.ATContentTypes.content.document import ATDocumentBase
from Products.ATContentTypes.content.image import ATCTImageTransform
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATNewsItem

from Products.ATContentTypes import ATCTMessageFactory as _

from Products.CMFCore.permissions import View

from Products.validation.config import validation
from Products.validation.validators.SupplValidators import MaxSizeValidator
from Products.validation import V_REQUIRED

validation.register(MaxSizeValidator('checkNewsImageMaxSize',
                                     maxsize=zconf.ATNewsItem.max_file_size))


ATNewsItemSchema = ATContentTypeSchema.copy() + Schema((
    TextField('text',
        required=False,
        searchable=True,
        primary=True,
        storage=AnnotationStorage(migrate=True),
        validators=('isTidyHtmlWithCleanup',),
        #validators=('isTidyHtml',),
        default_output_type='text/x-html-safe',
        widget=RichWidget(
            description='',
            label=_(u'label_body_text', u'Body Text'),
            rows=25,
            allow_file_upload=zconf.ATDocument.allow_document_upload)
        ),

    ImageField('image',
        required=False,
        storage=AnnotationStorage(migrate=True),
        languageIndependent=True,
        max_size=zconf.ATNewsItem.max_image_dimension,
        sizes={'large': (768, 768),
               'preview': (400, 400),
               'mini': (200, 200),
               'thumb': (128, 128),
               'tile': (64, 64),
               'icon': (32, 32),
               'listing': (16, 16),
              },
        validators=(('isNonEmptyFile', V_REQUIRED),
                    ('checkNewsImageMaxSize', V_REQUIRED)),
        widget=ImageWidget(
            description=_(u'help_news_image', default=u'Will be shown in the news listing, and in the news item itself. Image will be scaled to a sensible size.'),
            label=_(u'label_news_image', default=u'Image'),
            show_content_type=False)
        ),

    StringField('imageCaption',
        required=False,
        searchable=True,
        widget=StringWidget(
            description='',
            label=_(u'label_image_caption', default=u'Image Caption'),
            size=40)
        ),
    ), marshall=RFC822Marshaller()
    )

ATNewsItemSchema['description'].widget.label = \
    _(u'label_summary', default=u'Summary')

finalizeATCTSchema(ATNewsItemSchema)


class ATNewsItem(ATDocumentBase, ATCTImageTransform):
    """An announcement that will show up on the news portlet and in the news listing."""

    schema = ATNewsItemSchema

    portal_type = 'News Item'
    archetype_name = 'News Item'
    _atct_newTypeFor = {'portal_type': 'CMF News Item', 'meta_type': 'News Item'}
    assocMimetypes = ()
    assocFileExt = ('news', )
    cmf_edit_kws = ATDocumentBase.cmf_edit_kws

    implements(IATNewsItem)

    security = ClassSecurityInfo()

    security.declareProtected(View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        if 'title' not in kwargs:
            kwargs['title'] = self.getImageCaption()
        return self.getField('image').tag(self, **kwargs)

    security.declarePrivate('cmf_edit')
    def cmf_edit(self, text, description=None, text_format=None, **kwargs):
        if description is not None:
            self.setDescription(description)
        self.setText(text, mimetype=translateMimetypeAlias(text_format))
        self.update(**kwargs)

    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image

        return ATDocumentBase.__bobo_traverse__(self, REQUEST, name)

registerATCT(ATNewsItem, PROJECTNAME)
